@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: qbcfgtabpageaccountgen.cpp 902 2006-01-31 12:30:47Z cstim $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -381,8 +381,9 @@ void QBCfgTabPageAccountGeneral::slotLeftButtonClicked() {
   if (ul.empty()) {
     QMessageBox::critical(this,
                           tr("Selection Error"),
-                          tr("No user selected.\n"),
+                          tr("No user selected."),
                           QMessageBox::Retry,QMessageBox::NoButton);
+    return;
   }
   u=ul.front();
   _realPage->userList2->removeUser(u);
@@ -399,8 +400,9 @@ void QBCfgTabPageAccountGeneral::slotRightButtonClicked() {
   if (ul.empty()) {
     QMessageBox::critical(this,
                           tr("Selection Error"),
-                          tr("No user selected.\n"),
+                          tr("No user selected."),
                           QMessageBox::Retry,QMessageBox::NoButton);
+    return;
   }
   u=ul.front();
   _realPage->userList1->removeUser(u);
