@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: qbcfgtabpagebackends.h 809 2006-01-20 14:15:15Z cstim $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -42,9 +42,11 @@ public:
   virtual bool fromGui();
   virtual void updateView();
 
+signals:
+  void signalUpdate();
+
 public slots:
-  void slotBackendEnable();
-  void slotBackendDisable();
+  void slotUpdate();
 };
 
 
