@@ -1,13 +1,14 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: account.c 764 2006-01-13 14:00:00Z cstim $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -109,7 +110,7 @@ AB_USER_LIST2 *AB_Account_GetUsers(const AB_ACCOUNT *a) {
 
   ul=AB_User_List2_new();
   for (i=0; ; i++) {
-    GWEN_TYPE_UINT32 id;
+    uint32_t id;
     AB_USER *u;
 
     id=GWEN_DB_GetIntValue(a->data, "static/user", i, 0);
@@ -135,7 +136,7 @@ AB_USER_LIST2 *AB_Account_GetUsers(const AB_ACCOUNT *a) {
 
 
 AB_USER *AB_Account_GetFirstUser(const AB_ACCOUNT *a) {
-  GWEN_TYPE_UINT32 id;
+  uint32_t id;
   AB_USER *u;
 
   assert(a);
@@ -201,7 +202,7 @@ AB_USER_LIST2 *AB_Account_GetSelectedUsers(const AB_ACCOUNT *a) {
 
   ul=AB_User_List2_new();
   for (i=0; ; i++) {
-    GWEN_TYPE_UINT32 id;
+    uint32_t id;
     AB_USER *u;
 
     id=GWEN_DB_GetIntValue(a->data, "static/selectedUser", i, 0);
@@ -228,7 +229,7 @@ AB_USER_LIST2 *AB_Account_GetSelectedUsers(const AB_ACCOUNT *a) {
 
 
 AB_USER *AB_Account_GetFirstSelectedUser(const AB_ACCOUNT *a) {
-  GWEN_TYPE_UINT32 id;
+  uint32_t id;
   AB_USER *u;
 
   assert(a);
@@ -304,14 +305,18 @@ int AB_Account_toDb(const AB_ACCOUNT *a, GWEN_DB_NODE *db){
 
 void AB_Account_free(AB_ACCOUNT *a){
   if (a) {
-    if (--(a->usage)==0) {
+    assert(a->usage);
+    if (a->usage==1) {
       DBG_VERBOUS(AQBANKING_LOGDOMAIN, "Destroying AB_ACCOUNT");
-      GWEN_INHERIT_FINI(AB_ACCOUNT, a);
       GWEN_LIST_FINI(AB_ACCOUNT, a);
+      GWEN_INHERIT_FINI(AB_ACCOUNT, a);
       free(a->backendName);
       GWEN_DB_Group_free(a->data);
+      a->usage=0;
       GWEN_FREE_OBJECT(a);
     }
+    else
+      a->usage--;
   }
 }
 
@@ -324,7 +329,7 @@ void AB_Account_Attach(AB_ACCOUNT *a){
 
 
 
-GWEN_TYPE_UINT32 AB_Account_GetUniqueId(const AB_ACCOUNT *a){
+uint32_t AB_Account_GetUniqueId(const AB_ACCOUNT *a){
   assert(a);
   assert(a->usage);
   return GWEN_DB_GetIntValue(a->data, "static/uniqueId", 0, 0);
@@ -332,7 +337,7 @@ GWEN_TYPE_UINT32 AB_Account_GetUniqueId(const AB_ACCOUNT *a){
 
 
 
-void AB_Account_SetUniqueId(AB_ACCOUNT *a, GWEN_TYPE_UINT32 id){
+void AB_Account_SetUniqueId(AB_ACCOUNT *a, uint32_t id){
   assert(a);
   assert(a->usage);
   GWEN_DB_SetIntValue(a->data, GWEN_DB_FLAGS_OVERWRITE_VARS,
@@ -473,6 +478,46 @@ void AB_Account_SetBankName(AB_ACCOUNT *a, const char *s){
 
 
 
+const char *AB_Account_GetIBAN(const AB_ACCOUNT *a){
+  assert(a);
+  assert(a->usage);
+  return GWEN_DB_GetCharValue(a->data, "static/iban", 0, 0);
+}
+
+
+
+void AB_Account_SetIBAN(AB_ACCOUNT *a, const char *s){
+  assert(a);
+  assert(a->usage);
+  if (!s)
+    GWEN_DB_DeleteVar(a->data, "static/iban");
+  else
+    GWEN_DB_SetCharValue(a->data, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "static/iban", s);
+}
+
+
+
+const char *AB_Account_GetBIC(const AB_ACCOUNT *a){
+  assert(a);
+  assert(a->usage);
+  return GWEN_DB_GetCharValue(a->data, "static/bic", 0, 0);
+}
+
+
+
+void AB_Account_SetBIC(AB_ACCOUNT *a, const char *s){
+  assert(a);
+  assert(a->usage);
+  if (!s)
+    GWEN_DB_DeleteVar(a->data, "static/bic");
+  else
+    GWEN_DB_SetCharValue(a->data, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "static/bic", s);
+}
+
+
+
 const char *AB_Account_GetOwnerName(const AB_ACCOUNT *a){
   assert(a);
   assert(a->usage);
