@@ -1,13 +1,14 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: account.h 950 2006-02-22 19:39:02Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -82,10 +83,10 @@ AQBANKING_API
 void AB_Account_SetAccountType(AB_ACCOUNT *acc, AB_ACCOUNT_TYPE t);
 
 AQBANKING_API 
-GWEN_TYPE_UINT32 AB_Account_GetUniqueId(const AB_ACCOUNT *acc);
+uint32_t AB_Account_GetUniqueId(const AB_ACCOUNT *acc);
 
 AQBANKING_API
-void AB_Account_SetUniqueId(AB_ACCOUNT *acc, GWEN_TYPE_UINT32 id);
+void AB_Account_SetUniqueId(AB_ACCOUNT *acc, uint32_t id);
 
 AQBANKING_API 
 const char *AB_Account_GetBackendName(const AB_ACCOUNT *acc);
@@ -125,8 +126,20 @@ AQBANKING_API
 void AB_Account_SetBankName(AB_ACCOUNT *acc, const char *s);
 
 AQBANKING_API 
-const char *AB_Account_GetOwnerName(const AB_ACCOUNT *acc);
+const char *AB_Account_GetIBAN(const AB_ACCOUNT *a);
 AQBANKING_API 
+void AB_Account_SetIBAN(AB_ACCOUNT *a, const char *s);
+
+
+AQBANKING_API 
+const char *AB_Account_GetBIC(const AB_ACCOUNT *a);
+AQBANKING_API 
+void AB_Account_SetBIC(AB_ACCOUNT *a, const char *s);
+
+
+AQBANKING_API 
+const char *AB_Account_GetOwnerName(const AB_ACCOUNT *acc);
+AQBANKING_API
 void AB_Account_SetOwnerName(AB_ACCOUNT *acc, const char *s);
 
 AQBANKING_API 
