@@ -1,13 +1,14 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: bankinfoplugin_p.h 172 2004-12-05 21:42:43Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -22,7 +23,7 @@
 struct AB_BANKINFO_PLUGIN {
   GWEN_INHERIT_ELEMENT(AB_BANKINFO_PLUGIN);
   GWEN_LIST_ELEMENT(AB_BANKINFO_PLUGIN);
-  GWEN_TYPE_UINT32 usage;
+  uint32_t usage;
 
   char *country;
 
@@ -34,5 +35,14 @@ struct AB_BANKINFO_PLUGIN {
 };
 
 
+
+typedef struct AB_PLUGIN_BANKINFO AB_PLUGIN_BANKINFO;
+struct AB_PLUGIN_BANKINFO {
+  AB_PLUGIN_BANKINFO_FACTORY_FN pluginFactoryFn;
+};
+
+static void AB_Plugin_BankInfo_FreeData(void *bp, void *p);
+
+
 #endif /* AQBANKING_BANKINFOPLUGIN_P_H */
 
