@@ -1,13 +1,14 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: imexporter.c 1038 2006-07-03 03:01:30Z martin $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -18,6 +19,10 @@
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/inherit.h>
+#include <gwenhywfar/iolayer.h>
+#include <gwenhywfar/io_file.h>
+#include <gwenhywfar/io_memory.h>
+#include <gwenhywfar/iomanager.h>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -33,6 +38,9 @@ GWEN_INHERIT_FUNCTIONS(AB_IMEXPORTER)
 GWEN_LIST_FUNCTIONS(AB_IMEXPORTER, AB_ImExporter)
 GWEN_LIST_FUNCTIONS(AB_IMEXPORTER_ACCOUNTINFO, AB_ImExporterAccountInfo)
 
+GWEN_INHERIT(GWEN_PLUGIN, AB_PLUGIN_IMEXPORTER)
+
+
 
 
 AB_IMEXPORTER *AB_ImExporter_new(AB_BANKING *ab, const char *name){
@@ -69,47 +77,50 @@ void AB_ImExporter_free(AB_IMEXPORTER *ie){
 
 int AB_ImExporter_Import(AB_IMEXPORTER *ie,
                          AB_IMEXPORTER_CONTEXT *ctx,
-                         GWEN_BUFFEREDIO *bio,
-                         GWEN_DB_NODE *params){
+			 GWEN_IO_LAYER *io,
+			 GWEN_DB_NODE *params,
+			 uint32_t guiid){
   assert(ie);
   assert(ctx);
-  assert(bio);
+  assert(io);
   assert(params);
 
   if (ie->importFn)
-    return ie->importFn(ie, ctx, bio, params);
+    return ie->importFn(ie, ctx, io, params, guiid);
   else
-    return AB_ERROR_NOT_SUPPORTED;
+    return GWEN_ERROR_NOT_SUPPORTED;
 }
 
 
 
 int AB_ImExporter_Export(AB_IMEXPORTER *ie,
                          AB_IMEXPORTER_CONTEXT *ctx,
-                         GWEN_BUFFEREDIO *bio,
-                         GWEN_DB_NODE *params){
+			 GWEN_IO_LAYER *io,
+			 GWEN_DB_NODE *params,
+			 uint32_t guiid){
   assert(ie);
   assert(ctx);
-  assert(bio);
+  assert(io);
   assert(params);
 
   if (ie->exportFn)
-    return ie->exportFn(ie, ctx, bio, params);
+    return ie->exportFn(ie, ctx, io, params, guiid);
   else
-    return AB_ERROR_NOT_SUPPORTED;
+    return GWEN_ERROR_NOT_SUPPORTED;
 }
 
 
 
 int AB_ImExporter_CheckFile(AB_IMEXPORTER *ie,
-                            const char *fname){
+			    const char *fname,
+			    uint32_t guiid){
   assert(ie);
   assert(fname);
 
   if (ie->checkFileFn)
-    return ie->checkFileFn(ie, fname);
+    return ie->checkFileFn(ie, fname, guiid);
   else
-    return AB_ERROR_NOT_SUPPORTED;
+    return GWEN_ERROR_NOT_SUPPORTED;
 }
 
 
@@ -117,10 +128,11 @@ int AB_ImExporter_CheckFile(AB_IMEXPORTER *ie,
 int AB_ImExporter_ImportFile(AB_IMEXPORTER *ie,
                              AB_IMEXPORTER_CONTEXT *ctx,
                              const char *fname,
-                             GWEN_DB_NODE *dbProfile){
+			     GWEN_DB_NODE *dbProfile,
+			     uint32_t guiid){
   int fd;
-  GWEN_BUFFEREDIO *bio;
   int rv;
+  GWEN_IO_LAYER *io;
 
   assert(ie);
   assert(ctx);
@@ -131,14 +143,56 @@ int AB_ImExporter_ImportFile(AB_IMEXPORTER *ie,
   if (fd==-1) {
     /* error */
     DBG_ERROR(AQBANKING_LOGDOMAIN, "open(%s): %s", fname, strerror(errno));
-    return AB_ERROR_NOT_FOUND;
+    return GWEN_ERROR_NOT_FOUND;
   }
 
-  bio=GWEN_BufferedIO_File_new(fd);
-  GWEN_BufferedIO_SetReadBuffer(bio, 0, 1024);
-  rv=AB_ImExporter_Import(ie, ctx, bio, dbProfile);
-  GWEN_BufferedIO_Close(bio);
-  GWEN_BufferedIO_free(bio);
+  /* create io layer for this file (readonly) */
+  io=GWEN_Io_LayerFile_new(fd, -1);
+  assert(io);
+
+  rv=GWEN_Io_Manager_RegisterLayer(io);
+  if (rv) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
+    GWEN_Io_Layer_free(io);
+    return rv;
+  }
+
+  rv=AB_ImExporter_Import(ie, ctx, io, dbProfile, guiid);
+  GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, 2000);
+  GWEN_Io_Layer_free(io);
+
+  return rv;
+}
+
+
+
+int AB_ImExporter_ImportBuffer(AB_IMEXPORTER *ie,
+			       AB_IMEXPORTER_CONTEXT *ctx,
+			       GWEN_BUFFER *buf,
+			       GWEN_DB_NODE *dbProfile,
+			       uint32_t guiid) {
+  int rv;
+  GWEN_IO_LAYER *io;
+
+  assert(ie);
+  assert(ctx);
+  assert(buf);
+  assert(dbProfile);
+
+  /* create io layer for this file (readonly) */
+  io=GWEN_Io_LayerMemory_new(buf);
+  assert(io);
+
+  rv=GWEN_Io_Manager_RegisterLayer(io);
+  if (rv<0) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
+    GWEN_Io_Layer_free(io);
+    return rv;
+  }
+
+  rv=AB_ImExporter_Import(ie, ctx, io, dbProfile, guiid);
+  GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, 2000);
+  GWEN_Io_Layer_free(io);
 
   return rv;
 }
@@ -190,6 +244,10 @@ void AB_ImExporter_SetLibLoader(AB_IMEXPORTER *ie, GWEN_LIBLOADER *ll) {
 
 
 
+
+
+
+
 AB_IMEXPORTER_ACCOUNTINFO*
 AB_ImExporterContext_GetFirstAccountInfo(AB_IMEXPORTER_CONTEXT *iec){
   AB_IMEXPORTER_ACCOUNTINFO *ai;
@@ -246,6 +304,159 @@ AB_ImExporterContext_AccountInfoForEach(AB_IMEXPORTER_CONTEXT *iec,
 
 
 
+void AB_ImExporterContext_AddSecurity(AB_IMEXPORTER_CONTEXT *iec,
+				      AB_SECURITY *sec) {
+  assert(iec);
+  assert(sec);
+  AB_Security_List_Add(sec, iec->securityList);
+}
+
+
+
+AB_SECURITY*
+AB_ImExporterContext_GetFirstSecurity(AB_IMEXPORTER_CONTEXT *iec) {
+  AB_SECURITY *sec;
+
+  assert(iec);
+  sec=AB_Security_List_First(iec->securityList);
+  if (sec) {
+    iec->nextSecurity=AB_Security_List_Next(sec);
+    return sec;
+  }
+  iec->nextSecurity=NULL;
+  return NULL;
+}
+
+
+
+AB_SECURITY*
+AB_ImExporterContext_GetNextSecurity(AB_IMEXPORTER_CONTEXT *iec){
+  AB_SECURITY *sec;
+
+  assert(iec);
+  sec=iec->nextSecurity;
+  if (sec) {
+    iec->nextSecurity=AB_Security_List_Next(sec);
+    return sec;
+  }
+  iec->nextSecurity=NULL;
+  return NULL;
+}
+
+
+
+AB_SECURITY*
+AB_ImExporterContext_FindSecurity(AB_IMEXPORTER_CONTEXT *iec,
+				  const char *nameSpace,
+				  const char *id) {
+  AB_SECURITY *sec;
+
+  assert(iec);
+  if (nameSpace==NULL)
+    nameSpace="";
+  if (id==NULL)
+    id="";
+  sec=AB_Security_List_First(iec->securityList);
+  while(sec) {
+    const char *s1, *s2;
+
+    s1=AB_Security_GetUniqueId(sec);
+    if (s1==NULL)
+      s1="";
+    s2=AB_Security_GetNameSpace(sec);
+    if (s2==NULL)
+      s2="";
+    if (strcasecmp(s2, nameSpace)==0 &&
+	strcasecmp(s1, id)==0)
+      return sec;
+    sec=AB_Security_List_Next(sec);
+  }
+  return NULL;
+}
+
+
+
+void AB_ImExporterContext_AddMessage(AB_IMEXPORTER_CONTEXT *iec,
+				      AB_MESSAGE *msg) {
+  assert(iec);
+  assert(msg);
+  AB_Message_List_Add(msg, iec->messageList);
+}
+
+
+
+AB_MESSAGE*
+AB_ImExporterContext_GetFirstMessage(AB_IMEXPORTER_CONTEXT *iec) {
+  AB_MESSAGE *msg;
+
+  assert(iec);
+  msg=AB_Message_List_First(iec->messageList);
+  if (msg) {
+    iec->nextMessage=AB_Message_List_Next(msg);
+    return msg;
+  }
+  iec->nextMessage=NULL;
+  return NULL;
+}
+
+
+
+AB_MESSAGE*
+AB_ImExporterContext_GetNextMessage(AB_IMEXPORTER_CONTEXT *iec){
+  AB_MESSAGE *msg;
+
+  assert(iec);
+  msg=iec->nextMessage;
+  if (msg) {
+    iec->nextMessage=AB_Message_List_Next(msg);
+    return msg;
+  }
+  iec->nextMessage=NULL;
+  return NULL;
+}
+
+
+
+void AB_ImExporterContext_AddLog(AB_IMEXPORTER_CONTEXT *iec,
+				 const char *s) {
+  assert(iec);
+  if (s) {
+    size_t l=strlen(s);
+    if (s) {
+      GWEN_Buffer_AppendString(iec->logs, s);
+      if (s[l-1]!='\n')
+        GWEN_Buffer_AppendByte(iec->logs, '\n');
+    }
+  }
+}
+
+
+
+const char *AB_ImExporterContext_GetLog(const AB_IMEXPORTER_CONTEXT *iec) {
+  assert(iec);
+  if (GWEN_Buffer_GetUsedBytes(iec->logs))
+    return GWEN_Buffer_GetStart(iec->logs);
+  else
+    return NULL;
+}
+
+
+
+void AB_ImExporterContext_ClearLog(AB_IMEXPORTER_CONTEXT *iec) {
+  assert(iec);
+  GWEN_Buffer_Reset(iec->logs);
+}
+
+
+
+
+
+
+
+
+
+
+
 
 
 AB_IMEXPORTER_ACCOUNTINFO *AB_ImExporterAccountInfo_new() {
@@ -271,6 +482,7 @@ void AB_ImExporterAccountInfo_free(AB_IMEXPORTER_ACCOUNTINFO *iea){
     free(iea->accountNumber);
     free(iea->accountName);
     free(iea->iban);
+    free(iea->bic);
     free(iea->owner);
     free(iea->currency);
     free(iea->description);
@@ -287,6 +499,44 @@ void AB_ImExporterAccountInfo_free(AB_IMEXPORTER_ACCOUNTINFO *iea){
 
 
 
+void AB_ImExporterAccountInfo_FillFromAccount(AB_IMEXPORTER_ACCOUNTINFO *iea,
+					      const AB_ACCOUNT *a) {
+  const char *s;
+  AB_ACCOUNT_TYPE at;
+
+  assert(iea);
+  assert(a);
+
+  s=AB_Account_GetBankCode(a);
+  AB_ImExporterAccountInfo_SetBankCode(iea, s);
+
+  s=AB_Account_GetBankName(a);
+  AB_ImExporterAccountInfo_SetBankName(iea, s);
+
+  s=AB_Account_GetAccountNumber(a);
+  AB_ImExporterAccountInfo_SetAccountNumber(iea, s);
+
+  s=AB_Account_GetAccountName(a);
+  AB_ImExporterAccountInfo_SetAccountName(iea, s);
+
+  s=AB_Account_GetIBAN(a);
+  AB_ImExporterAccountInfo_SetIban(iea, s);
+
+  s=AB_Account_GetBIC(a);
+  AB_ImExporterAccountInfo_SetBic(iea, s);
+
+  s=AB_Account_GetCurrency(a);
+  AB_ImExporterAccountInfo_SetCurrency(iea, s);
+
+  s=AB_Account_GetOwnerName(a);
+  AB_ImExporterAccountInfo_SetOwner(iea, s);
+
+  at=AB_Account_GetAccountType(a);
+  AB_ImExporterAccountInfo_SetType(iea, at);
+}
+
+
+
 AB_IMEXPORTER_ACCOUNTINFO*
 AB_ImExporterAccountInfo_dup(const AB_IMEXPORTER_ACCOUNTINFO *oi) {
   AB_IMEXPORTER_ACCOUNTINFO *iea;
@@ -302,6 +552,7 @@ AB_ImExporterAccountInfo_dup(const AB_IMEXPORTER_ACCOUNTINFO *oi) {
   COPY_CHAR(accountNumber);
   COPY_CHAR(accountName);
   COPY_CHAR(iban);
+  COPY_CHAR(bic);
   COPY_CHAR(owner);
   COPY_CHAR(currency);
   COPY_CHAR(description);
@@ -336,6 +587,7 @@ int AB_ImExporterAccountInfo_toDb(const AB_IMEXPORTER_ACCOUNTINFO *iea,
   STORE_CHAR(accountNumber);
   STORE_CHAR(accountName);
   STORE_CHAR(iban);
+  STORE_CHAR(bic);
   STORE_CHAR(owner);
   STORE_CHAR(currency);
   STORE_CHAR(description);
@@ -510,6 +762,7 @@ AB_ImExporterAccountInfo_fromDb(GWEN_DB_NODE *db){
   RESTORE_CHAR(bankName);
   RESTORE_CHAR(accountNumber);
   RESTORE_CHAR(iban);
+  RESTORE_CHAR(bic);
   RESTORE_CHAR(owner);
   RESTORE_CHAR(currency);
   RESTORE_CHAR(description);
@@ -915,6 +1168,24 @@ void AB_ImExporterAccountInfo_SetIban(AB_IMEXPORTER_ACCOUNTINFO *iea,
 
 
 
+const char*
+AB_ImExporterAccountInfo_GetBic(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return iea->bic;
+}
+
+
+
+void AB_ImExporterAccountInfo_SetBic(AB_IMEXPORTER_ACCOUNTINFO *iea,
+				     const char *s) {
+  assert(iea);
+  free(iea->bic);
+  if (s) iea->bic=strdup(s);
+  else iea->bic=0;
+}
+
+
+
 const char*
 AB_ImExporterAccountInfo_GetCurrency(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
   assert(iea);
@@ -1035,7 +1306,6 @@ AB_ImExporterAccountInfo_GetNextAccountStatus(AB_IMEXPORTER_ACCOUNTINFO *iea){
   t=iea->nextAccountStatus;
   if (t) {
     iea->nextAccountStatus=AB_AccountStatus_List_Next(t);
-    AB_AccountStatus_List_Del(t);
     return t;
   }
   iea->nextAccountStatus=0;
@@ -1058,6 +1328,9 @@ AB_IMEXPORTER_CONTEXT *AB_ImExporterContext_new(){
 
   GWEN_NEW_OBJECT(AB_IMEXPORTER_CONTEXT, iec);
   iec->accountInfoList=AB_ImExporterAccountInfo_List_new();
+  iec->securityList=AB_Security_List_new();
+  iec->messageList=AB_Message_List_new();
+  iec->logs=GWEN_Buffer_new(0, 128, 0, 1);
 
   return iec;
 }
@@ -1066,6 +1339,9 @@ AB_IMEXPORTER_CONTEXT *AB_ImExporterContext_new(){
 
 void AB_ImExporterContext_free(AB_IMEXPORTER_CONTEXT *iec){
   if (iec) {
+    GWEN_Buffer_free(iec->logs);
+    AB_Message_List_free(iec->messageList);
+    AB_Security_List_free(iec->securityList);
     AB_ImExporterAccountInfo_List_free(iec->accountInfoList);
     GWEN_FREE_OBJECT(iec);
   }
@@ -1076,6 +1352,8 @@ void AB_ImExporterContext_free(AB_IMEXPORTER_CONTEXT *iec){
 int AB_ImExporterContext_toDb(const AB_IMEXPORTER_CONTEXT *iec,
 			       GWEN_DB_NODE *db){
   AB_IMEXPORTER_ACCOUNTINFO *iea;
+  AB_SECURITY *sec;
+  AB_MESSAGE *msg;
 
   iea=AB_ImExporterAccountInfo_List_First(iec->accountInfoList);
   if (iea) {
@@ -1098,6 +1376,63 @@ int AB_ImExporterContext_toDb(const AB_IMEXPORTER_CONTEXT *iec,
     }
   }
 
+  sec=AB_Security_List_First(iec->securityList);
+  if (sec) {
+    GWEN_DB_NODE *dbG;
+
+    dbG=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
+			 "securityList");
+    assert(dbG);
+
+    while(sec) {
+      GWEN_DB_NODE *dbT;
+
+      dbT=GWEN_DB_GetGroup(dbG, GWEN_PATH_FLAGS_CREATE_GROUP,
+			   "security");
+      assert(dbT);
+
+      if (AB_Security_toDb(sec, dbT))
+	return -1;
+      sec=AB_Security_List_Next(sec);
+    }
+  }
+
+  msg=AB_Message_List_First(iec->messageList);
+  if (msg) {
+    GWEN_DB_NODE *dbG;
+
+    dbG=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
+			 "messageList");
+    assert(dbG);
+
+    while(msg) {
+      GWEN_DB_NODE *dbT;
+
+      dbT=GWEN_DB_GetGroup(dbG, GWEN_PATH_FLAGS_CREATE_GROUP,
+			   "message");
+      assert(dbT);
+
+      if (AB_Message_toDb(msg, dbT))
+	return -1;
+      msg=AB_Message_List_Next(msg);
+    }
+  }
+
+  GWEN_DB_DeleteVar(db, "logs");
+  if (GWEN_Buffer_GetUsedBytes(iec->logs)) {
+    const char *s;
+
+    s=GWEN_Buffer_GetStart(iec->logs);
+    while(*s) {
+      char buf[64];
+
+      buf[0]=0;
+      strncat(buf, s, sizeof(buf));
+      GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, "logs", buf);
+      s+=strlen(buf);
+    }
+  }
+
   return 0;
 }
 
@@ -1106,6 +1441,7 @@ int AB_ImExporterContext_toDb(const AB_IMEXPORTER_CONTEXT *iec,
 int AB_ImExporterContext_ReadDb(AB_IMEXPORTER_CONTEXT *iec,
                                 GWEN_DB_NODE *db) {
   GWEN_DB_NODE *dbT;
+  int i;
 
   dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
 		       "accountInfoList");
@@ -1121,6 +1457,41 @@ int AB_ImExporterContext_ReadDb(AB_IMEXPORTER_CONTEXT *iec,
     }
   }
 
+  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+		       "securityList");
+  if (dbT) {
+    dbT=GWEN_DB_FindFirstGroup(dbT, "security");
+    while(dbT) {
+      AB_SECURITY *sec;
+
+      sec=AB_Security_fromDb(dbT);
+      assert(sec);
+      AB_Security_List_Add(sec, iec->securityList);
+      dbT=GWEN_DB_FindNextGroup(dbT, "security");
+    }
+  }
+
+  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+		       "messageList");
+  if (dbT) {
+    dbT=GWEN_DB_FindFirstGroup(dbT, "message");
+    while(dbT) {
+      AB_MESSAGE *msg;
+
+      msg=AB_Message_fromDb(dbT);
+      assert(msg);
+      AB_Message_List_Add(msg, iec->messageList);
+      dbT=GWEN_DB_FindNextGroup(dbT, "message");
+    }
+  }
+  for (i=0; ; i++) {
+    const char *s;
+
+    s=GWEN_DB_GetCharValue(db, "logs", i, NULL);
+    if (!s)
+      break;
+    GWEN_Buffer_AppendString(iec->logs, s);
+  }
   return 0;
 }
 
@@ -1254,6 +1625,51 @@ void AB_ImExporterContext_AddTransaction(AB_IMEXPORTER_CONTEXT *iec,
 
 
 
+void AB_ImExporterContext_AddTransfer(AB_IMEXPORTER_CONTEXT *iec,
+				      AB_TRANSACTION *t){
+  AB_IMEXPORTER_ACCOUNTINFO *iea;
+
+  iea=AB_ImExporterContext_GetAccountInfo
+    (iec,
+     AB_Transaction_GetLocalBankCode(t),
+     AB_Transaction_GetLocalAccountNumber(t)
+    );
+  assert(iea);
+  AB_ImExporterAccountInfo_AddTransfer(iea, t);
+}
+
+
+
+void AB_ImExporterContext_AddDatedTransfer(AB_IMEXPORTER_CONTEXT *iec,
+					   AB_TRANSACTION *t){
+  AB_IMEXPORTER_ACCOUNTINFO *iea;
+
+  iea=AB_ImExporterContext_GetAccountInfo
+    (iec,
+     AB_Transaction_GetLocalBankCode(t),
+     AB_Transaction_GetLocalAccountNumber(t)
+    );
+  assert(iea);
+  AB_ImExporterAccountInfo_AddDatedTransfer(iea, t);
+}
+
+
+
+void AB_ImExporterContext_AddStandingOrder(AB_IMEXPORTER_CONTEXT *iec,
+					   AB_TRANSACTION *t){
+  AB_IMEXPORTER_ACCOUNTINFO *iea;
+
+  iea=AB_ImExporterContext_GetAccountInfo
+    (iec,
+     AB_Transaction_GetLocalBankCode(t),
+     AB_Transaction_GetLocalAccountNumber(t)
+    );
+  assert(iea);
+  AB_ImExporterAccountInfo_AddStandingOrder(iea, t);
+}
+
+
+
 
 void AB_ImExporter_Utf8ToDta(const char *p,
                              int size,
@@ -1359,7 +1775,6 @@ GWEN_TIME *AB_ImExporter_DateFromString(const char *p, const char *tmpl,
 
     ti=GWEN_Time_fromUtcString(GWEN_Buffer_GetStart(dbuf),
 			       GWEN_Buffer_GetStart(tbuf));
-    assert(ti);
     GWEN_Buffer_free(tbuf);
     GWEN_Buffer_free(dbuf);
   }
@@ -1383,6 +1798,8 @@ void AB_ImExporter_Iso8859_1ToUtf8(const char *p,
       break;
 
     c=(unsigned char)(*(p++));
+    if (c<32 || c==127)
+      c=32;
     switch(c) {
     case 0xc4: /* AE */
       GWEN_Buffer_AppendByte(buf, 0xc3);
@@ -1531,7 +1948,7 @@ int AH_ImExporter__Transform_Var(GWEN_DB_NODE *db, int level) {
 
   dbC=GWEN_DB_GetFirstValue(db);
   while(dbC) {
-    if (GWEN_DB_GetValueType(dbC)==GWEN_DB_VALUETYPE_CHAR) {
+    if (GWEN_DB_GetValueType(dbC)==GWEN_DB_NodeType_ValueChar) {
       const char *s;
       unsigned int l;
 
@@ -1611,6 +2028,8 @@ void AB_ImExporterContext_AddContext(AB_IMEXPORTER_CONTEXT *iec,
     AB_ImExporterAccountInfo_List_Add(iea, iec->accountInfoList);
     iea=nextIea;
   }
+
+  GWEN_Buffer_AppendBuffer(iec->logs, toAdd->logs);
   AB_ImExporterContext_free(toAdd);
 }
 
@@ -1618,3 +2037,61 @@ void AB_ImExporterContext_AddContext(AB_IMEXPORTER_CONTEXT *iec,
 
 
 
+
+
+
+
+GWEN_PLUGIN *AB_Plugin_ImExporter_new(GWEN_PLUGIN_MANAGER *pm,
+				      const char *name,
+				      const char *fileName) {
+  GWEN_PLUGIN *pl;
+  AB_PLUGIN_IMEXPORTER *xpl;
+
+  pl=GWEN_Plugin_new(pm, name, fileName);
+  GWEN_NEW_OBJECT(AB_PLUGIN_IMEXPORTER, xpl);
+  GWEN_INHERIT_SETDATA(GWEN_PLUGIN, AB_PLUGIN_IMEXPORTER, pl, xpl,
+		       AB_Plugin_ImExporter_FreeData);
+
+  return pl;
+}
+
+
+
+void AB_Plugin_ImExporter_FreeData(void *bp, void *p) {
+  AB_PLUGIN_IMEXPORTER *xpl;
+
+  xpl=(AB_PLUGIN_IMEXPORTER*)p;
+  GWEN_FREE_OBJECT(xpl);
+}
+
+
+
+AB_IMEXPORTER *AB_Plugin_ImExporter_Factory(GWEN_PLUGIN *pl,
+					    AB_BANKING *ab,
+					    GWEN_DB_NODE *db) {
+  AB_PLUGIN_IMEXPORTER *xpl;
+
+  assert(pl);
+  xpl=GWEN_INHERIT_GETDATA(GWEN_PLUGIN, AB_PLUGIN_IMEXPORTER, pl);
+  assert(xpl);
+
+  assert(xpl->pluginFactoryFn);
+  return xpl->pluginFactoryFn(pl, ab, db);
+}
+
+
+void AB_Plugin_ImExporter_SetFactoryFn(GWEN_PLUGIN *pl,
+				       AB_PLUGIN_IMEXPORTER_FACTORY_FN fn) {
+  AB_PLUGIN_IMEXPORTER *xpl;
+
+  assert(pl);
+  xpl=GWEN_INHERIT_GETDATA(GWEN_PLUGIN, AB_PLUGIN_IMEXPORTER, pl);
+  assert(xpl);
+
+  xpl->pluginFactoryFn=fn;
+}
+
+
+
+
+
