@@ -1,13 +1,14 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: imexporter.h 1038 2006-07-03 03:01:30Z martin $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -15,7 +16,7 @@
 #define AQBANKING_IMEXPORTER_H
 
 #include <gwenhywfar/inherit.h>
-#include <gwenhywfar/bufferedio.h>
+#include <gwenhywfar/iolayer.h>
 #include <gwenhywfar/db.h>
 #include <gwenhywfar/types.h>
 #include <aqbanking/error.h>
@@ -58,6 +59,8 @@ typedef struct AB_IMEXPORTER_ACCOUNTINFO AB_IMEXPORTER_ACCOUNTINFO;
 #include <aqbanking/banking.h>
 #include <aqbanking/account.h>
 #include <aqbanking/transaction.h>
+#include <aqbanking/security.h>
+#include <aqbanking/message.h>
 
 
 #ifdef __cplusplus
@@ -83,8 +86,9 @@ extern "C" {
 AQBANKING_API 
 int AB_ImExporter_Import(AB_IMEXPORTER *ie,
                          AB_IMEXPORTER_CONTEXT *ctx,
-                         GWEN_BUFFEREDIO *bio,
-                         GWEN_DB_NODE *dbProfile);
+			 GWEN_IO_LAYER *io,
+			 GWEN_DB_NODE *dbProfile,
+			 uint32_t guiid);
 
 /**
  * Writes all data to the given stream.
@@ -98,8 +102,9 @@ int AB_ImExporter_Import(AB_IMEXPORTER *ie,
 AQBANKING_API 
 int AB_ImExporter_Export(AB_IMEXPORTER *ie,
                          AB_IMEXPORTER_CONTEXT *ctx,
-                         GWEN_BUFFEREDIO *bio,
-                         GWEN_DB_NODE *dbProfile);
+			 GWEN_IO_LAYER *io,
+			 GWEN_DB_NODE *dbProfile,
+			 uint32_t guiid);
 
 /**
  * This is just a convenience function for @ref AB_ImExporter_Import.
@@ -108,14 +113,24 @@ AQBANKING_API
 int AB_ImExporter_ImportFile(AB_IMEXPORTER *ie,
                              AB_IMEXPORTER_CONTEXT *ctx,
                              const char *fname,
-                             GWEN_DB_NODE *dbProfile);
+			     GWEN_DB_NODE *dbProfile,
+			     uint32_t guiid);
+
+AQBANKING_API
+int AB_ImExporter_ImportBuffer(AB_IMEXPORTER *ie,
+			       AB_IMEXPORTER_CONTEXT *ctx,
+                               GWEN_BUFFER *buf,
+			       GWEN_DB_NODE *dbProfile,
+			       uint32_t guiid);
+
 
 /**
  * This function checks whether the given importer supports the given file.
  */
 AQBANKING_API
 int AB_ImExporter_CheckFile(AB_IMEXPORTER *ie,
-			    const char *fname);
+			    const char *fname,
+			    uint32_t guiid);
 
 /*@}*/
 
@@ -273,6 +288,100 @@ AB_ImExporterContext_GetAccountInfo(AB_IMEXPORTER_CONTEXT *iec,
 AQBANKING_API
 void AB_ImExporterContext_AddTransaction(AB_IMEXPORTER_CONTEXT *iec,
                                          AB_TRANSACTION *t);
+
+
+/**
+ * This is just a convenience function. It takes the bank code and
+ * account number from the account, and then calls
+ * @ref AB_ImExporterContext_GetAccountInfo and
+ * @ref AB_ImExporterAccountInfo_AddTransfer.
+ * If you want to add many transfers which are sorted by account
+ * it is much faster to avoid this function and to select the appropriate
+ * account info object once before importing all transactions for this
+ * particular account. This would save you the additional lookup before
+ * every transaction.
+ */
+AQBANKING_API
+void AB_ImExporterContext_AddTransfer(AB_IMEXPORTER_CONTEXT *iec,
+				      AB_TRANSACTION *t);
+
+
+AQBANKING_API
+void AB_ImExporterContext_AddStandingOrder(AB_IMEXPORTER_CONTEXT *iec,
+					   AB_TRANSACTION *t);
+
+
+AQBANKING_API
+void AB_ImExporterContext_AddDatedTransfer(AB_IMEXPORTER_CONTEXT *iec,
+					   AB_TRANSACTION *t);
+
+
+
+/** @defgroup AB_IMEXPORTER_CONTEXT_SECURITY Securities
+ *
+ * These functions handle stocks, mutual funds etc.
+ */
+/**@{*/
+AQBANKING_API 
+void AB_ImExporterContext_AddSecurity(AB_IMEXPORTER_CONTEXT *iec,
+				      AB_SECURITY *sec);
+
+AQBANKING_API 
+AB_SECURITY*
+AB_ImExporterContext_GetFirstSecurity(AB_IMEXPORTER_CONTEXT *iec);
+
+AQBANKING_API 
+AB_SECURITY*
+AB_ImExporterContext_GetNextSecurity(AB_IMEXPORTER_CONTEXT *iec);
+
+AQBANKING_API 
+AB_SECURITY*
+AB_ImExporterContext_FindSecurity(AB_IMEXPORTER_CONTEXT *iec,
+				  const char *nameSpace,
+				  const char *id);
+
+/**@}*/
+
+
+
+/** @defgroup AB_IMEXPORTER_CONTEXT_MESSAGE Messages
+ *
+ * These functions handle messages received from a bank.
+ */
+/**@{*/
+AQBANKING_API 
+void AB_ImExporterContext_AddMessage(AB_IMEXPORTER_CONTEXT *iec,
+				     AB_MESSAGE *msg);
+
+AQBANKING_API 
+AB_MESSAGE*
+AB_ImExporterContext_GetFirstMessage(AB_IMEXPORTER_CONTEXT *iec);
+
+AQBANKING_API 
+AB_MESSAGE*
+AB_ImExporterContext_GetNextMessage(AB_IMEXPORTER_CONTEXT *iec);
+
+/**@}*/
+
+/** @defgroup AB_IMEXPORTER_CONTEXT_LOGS Logs
+ *
+ * These functions handle logs written by the backends.
+ * Such logs are meant to be read and interpreted by a human user.
+ */
+/**@{*/
+
+AQBANKING_API
+void AB_ImExporterContext_AddLog(AB_IMEXPORTER_CONTEXT *iec,
+				 const char *s);
+
+AQBANKING_API
+const char *AB_ImExporterContext_GetLog(const AB_IMEXPORTER_CONTEXT *iec);
+
+AQBANKING_API
+void AB_ImExporterContext_ClearLog(AB_IMEXPORTER_CONTEXT *iec);
+
+/**@}*/
+
 /*@}*/ /* defgroup */
 
 
@@ -295,6 +404,10 @@ AB_IMEXPORTER_ACCOUNTINFO *AB_ImExporterAccountInfo_new();
 AQBANKING_API 
 void AB_ImExporterAccountInfo_free(AB_IMEXPORTER_ACCOUNTINFO *iea);
 
+AQBANKING_API 
+void AB_ImExporterAccountInfo_FillFromAccount(AB_IMEXPORTER_ACCOUNTINFO *iea,
+					      const AB_ACCOUNT *a);
+
 /**
  * Returns a copy of the given account info. Please note that only the data
  * is copied, internal pointers for
@@ -363,6 +476,17 @@ AQBANKING_API
 void AB_ImExporterAccountInfo_SetIban(AB_IMEXPORTER_ACCOUNTINFO *iea,
                                       const char *s);
 
+
+/**
+ * BIC.
+ */
+AQBANKING_API 
+const char*
+AB_ImExporterAccountInfo_GetBic(const AB_IMEXPORTER_ACCOUNTINFO *iea);
+AQBANKING_API 
+void AB_ImExporterAccountInfo_SetBic(AB_IMEXPORTER_ACCOUNTINFO *iea,
+				     const char *s);
+
 /**
  * Account currency
  */
@@ -489,9 +613,7 @@ void AB_ImExporterAccountInfo_AddAccountStatus(AB_IMEXPORTER_ACCOUNTINFO *iea,
 /**
  * Returns the first account status stored within the context and removes
  * it.
- * The caller becomes the new owner of the account status returned (if any)
- * which makes him/her responsible for freeing it using
- * @ref AB_AccountStatus_free.
+ * The context remains the owner of the object returned.
  */
 AQBANKING_API 
 AB_ACCOUNT_STATUS*
@@ -499,9 +621,7 @@ AB_ImExporterAccountInfo_GetFirstAccountStatus(AB_IMEXPORTER_ACCOUNTINFO *iea);
 
 /**
  * Returns the next account status stored within the context and removes it
- * The caller becomes the new owner of the account status returned (if any)
- * which makes him/her responsible for freeing it using
- * @ref AB_AccountStatus_free.
+ * The context remains the owner of the object returned.
  */
 AQBANKING_API 
 AB_ACCOUNT_STATUS*
@@ -781,6 +901,7 @@ GWEN_TIME *AB_ImExporter_DateFromString(const char *p,
                                         const char *tmpl,
                                         int inUtc);
 
+
 /*@}*/
 
 
