@@ -1,13 +1,14 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: provider_be.h 953 2006-02-24 03:07:59Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 /** @file provider_be.h
@@ -21,6 +22,8 @@
 #include <aqbanking/provider.h>
 #include <aqbanking/user.h>
 
+#include <gwenhywfar/plugin.h>
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -32,21 +35,6 @@ extern "C" {
  */
 /*@{*/
 
-/**
- * <p>
- * This is the prototype for the provider factory function expected in
- * provider plugins.
- * Every provider plugin must have at least this function which returns
- * a new provider object.
- * </p>
- * <p>
- * This function must contain the name of the provider in all-lowercase
- * letters. For AqHBCI this function is called "aqhbci_factory".
- * </p>
- * @param ab AqBanking main object
- */
-typedef AB_PROVIDER* (*AB_PROVIDER_FACTORY_FN)(AB_BANKING *ab);
-
 /**
  * This type is used with @ref AB_Provider_ExtendAccount and
  * @ref AB_Provider_ExtendUser.
@@ -91,18 +79,21 @@ typedef int (*AB_PROVIDER_FINI_FN)(AB_PROVIDER *pro, GWEN_DB_NODE *dbData);
 /**
  * See @ref AB_Provider_UpdateJob
  */
-typedef int (*AB_PROVIDER_UPDATEJOB_FN)(AB_PROVIDER *pro, AB_JOB *j);
+typedef int (*AB_PROVIDER_UPDATEJOB_FN)(AB_PROVIDER *pro, AB_JOB *j,
+					uint32_t guiid);
 
 /**
  * See @ref AB_Provider_AddJob.
  */
-typedef int (*AB_PROVIDER_ADDJOB_FN)(AB_PROVIDER *pro, AB_JOB *j);
+typedef int (*AB_PROVIDER_ADDJOB_FN)(AB_PROVIDER *pro, AB_JOB *j,
+				     uint32_t guiid);
 
 /**
  * See @ref AB_Provider_Execute
  */
 typedef int (*AB_PROVIDER_EXECUTE_FN)(AB_PROVIDER *pro,
-                                      AB_IMEXPORTER_CONTEXT *ctx);
+				      AB_IMEXPORTER_CONTEXT *ctx,
+				      uint32_t guiid);
 
 
 /**
@@ -126,8 +117,8 @@ typedef int (*AB_PROVIDER_EXTEND_ACCOUNT_FN)(AB_PROVIDER *pro,
                                              AB_PROVIDER_EXTEND_MODE um);
 
 typedef int (*AB_PROVIDER_UPDATE_FN)(AB_PROVIDER *pro,
-                                     GWEN_TYPE_UINT32 lastVersion,
-                                     GWEN_TYPE_UINT32 currentVersion);
+                                     uint32_t lastVersion,
+                                     uint32_t currentVersion);
 
 
 /*@}*/
@@ -185,7 +176,7 @@ int AB_Provider_Fini(AB_PROVIDER *pro);
  * @param j job to update
  */
 AQBANKING_API
-int AB_Provider_UpdateJob(AB_PROVIDER *pro, AB_JOB *j);
+int AB_Provider_UpdateJob(AB_PROVIDER *pro, AB_JOB *j, uint32_t guiid);
 
 /**
  * <p>
@@ -209,7 +200,7 @@ int AB_Provider_UpdateJob(AB_PROVIDER *pro, AB_JOB *j);
  * @param pro backend object
  */
 AQBANKING_API
-int AB_Provider_AddJob(AB_PROVIDER *pro, AB_JOB *j);
+int AB_Provider_AddJob(AB_PROVIDER *pro, AB_JOB *j, uint32_t guiid);
 
 /**
  * Executes all jobs in the queue which have just been added via
@@ -219,7 +210,8 @@ int AB_Provider_AddJob(AB_PROVIDER *pro, AB_JOB *j);
  * @param ctx im-/exporter context to receive responses
  */
 AQBANKING_API
-int AB_Provider_Execute(AB_PROVIDER *pro, AB_IMEXPORTER_CONTEXT *ctx);
+int AB_Provider_Execute(AB_PROVIDER *pro, AB_IMEXPORTER_CONTEXT *ctx,
+			uint32_t guiid);
 
 /**
  * Resets the queue of the backend.
@@ -257,8 +249,8 @@ int AB_Provider_ExtendAccount(AB_PROVIDER *pro, AB_ACCOUNT *a,
  */
 AQBANKING_API
 int AB_Provider_Update(AB_PROVIDER *pro,
-                       GWEN_TYPE_UINT32 lastVersion,
-                       GWEN_TYPE_UINT32 currentVersion);
+                       uint32_t lastVersion,
+                       uint32_t currentVersion);
 
 
 /*@}*/
@@ -298,6 +290,28 @@ void AB_Provider_SetUpdateFn(AB_PROVIDER *pro, AB_PROVIDER_UPDATE_FN f);
 /*@}*/
 
 
+
+
+typedef AB_PROVIDER* (*AB_PLUGIN_PROVIDER_FACTORY_FN)(GWEN_PLUGIN *pl,
+						      AB_BANKING *ab);
+
+
+AQBANKING_API
+GWEN_PLUGIN *AB_Plugin_Provider_new(GWEN_PLUGIN_MANAGER *pm,
+				    const char *name,
+				    const char *fileName);
+
+
+AQBANKING_API
+AB_PROVIDER *AB_Plugin_Provider_Factory(GWEN_PLUGIN *pl, AB_BANKING *ab);
+
+AQBANKING_API
+void AB_Plugin_Provider_SetFactoryFn(GWEN_PLUGIN *pl,
+				     AB_PLUGIN_PROVIDER_FACTORY_FN fn);
+
+
+
+
 /*@}*/ /* defgroup */
 
 
