@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: provider_l.h 950 2006-02-22 19:39:02Z aquamaniac $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -20,5 +20,15 @@
 AH_HBCI *AH_Provider_GetHbci(const AB_PROVIDER *pro);
 
 
+AQHBCI_API
+int AH_Provider_SendDtazv(AB_PROVIDER *pro,
+			  AB_ACCOUNT *a,
+			  AB_IMEXPORTER_CONTEXT *ctx,
+			  const uint8_t *dataPtr,
+			  uint32_t dataLen,
+			  int nounmount,
+			  uint32_t guiid);
+
+
 #endif
 
