@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: account_p.h 764 2006-01-13 14:00:00Z cstim $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -22,10 +22,10 @@
 typedef struct AH_ACCOUNT AH_ACCOUNT;
 struct AH_ACCOUNT {
   AH_HBCI *hbci;
-  GWEN_TYPE_UINT32 flags;
+  uint32_t flags;
 };
 
-static void AH_Account_freeData(void *bp, void *p);
+static void GWENHYWFAR_CB AH_Account_freeData(void *bp, void *p);
 
 
 #endif /* AH_ACCOUNT_P_H */
