@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: createkeys.c 964 2006-03-17 10:35:21Z cstim $
+ cvs         : $Id$
  begin       : Tue May 03 2005
  copyright   : (C) 2005 by Martin Preuss
  email       : martin@libchipcard.de
@@ -42,7 +42,7 @@ int createKeys(AB_BANKING *ab,
   const GWEN_ARGS args[]={
   {
     GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-    GWEN_ArgsTypeChar,            /* type */
+    GWEN_ArgsType_Char,           /* type */
     "bankId",                     /* name */
     0,                            /* minnum */
     1,                            /* maxnum */
@@ -53,7 +53,7 @@ int createKeys(AB_BANKING *ab,
   },
   {
     GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-    GWEN_ArgsTypeChar,            /* type */
+    GWEN_ArgsType_Char,           /* type */
     "customerId",                 /* name */
     0,                            /* minnum */
     1,                            /* maxnum */
@@ -64,7 +64,7 @@ int createKeys(AB_BANKING *ab,
   },
   {
     GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
-    GWEN_ArgsTypeInt,             /* type */
+    GWEN_ArgsType_Int,            /* type */
     "help",                       /* name */
     0,                            /* minnum */
     0,                            /* maxnum */
@@ -88,7 +88,7 @@ int createKeys(AB_BANKING *ab,
     GWEN_BUFFER *ubuf;
 
     ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
-    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutTypeTXT)) {
+    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
       fprintf(stderr, "ERROR: Could not create help string\n");
       return 1;
     }
@@ -103,6 +103,12 @@ int createKeys(AB_BANKING *ab,
     return 2;
   }
 
+  rv=AB_Banking_OnlineInit(ab);
+  if (rv) {
+    DBG_ERROR(0, "Error on init (%d)", rv);
+    return 2;
+  }
+
   pro=AB_Banking_GetProvider(ab, "aqhbci");
   assert(pro);
 
@@ -134,42 +140,19 @@ int createKeys(AB_BANKING *ab,
     return 3;
   }
   else {
-    AH_MEDIUM *m;
-
-    m=AH_User_GetMedium(u);
-    assert(m);
-
-    rv=AH_Medium_Mount(m);
+    rv=AH_Provider_CreateKeys(pro, u, 0, 0);
     if (rv) {
-      DBG_ERROR(0, "Could not mount medium (%d)", rv);
-      AB_Banking_Fini(ab);
-      return 3;
-    }
-
-    rv=AH_Medium_SelectContext(m, AH_User_GetContextIdx(u));
-    if (rv) {
-      DBG_ERROR(0, "Could not select context %d (%d)",
-                AH_User_GetContextIdx(u), rv);
-      AB_Banking_Fini(ab);
-      return 3;
-    }
-
-    fprintf(stderr, "Creating keys, please wait...\n");
-    rv=AH_Medium_CreateKeys(m);
-    if (rv) {
-      DBG_ERROR(0, "Could not generate keys (%d)", rv);
-      AB_Banking_Fini(ab);
-      return 3;
-    }
-
-    rv=AH_Medium_Unmount(m, 1);
-    if (rv) {
-      DBG_ERROR(0, "Could not unmount medium (%d)", rv);
+      DBG_ERROR(0, "Error creating keys (%d)", rv);
       AB_Banking_Fini(ab);
       return 3;
     }
   }
 
+  rv=AB_Banking_OnlineFini(ab);
+  if (rv) {
+    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
+    return 5;
+  }
 
   rv=AB_Banking_Fini(ab);
   if (rv) {
