@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: getkeys.c 964 2006-03-17 10:35:21Z cstim $
+ cvs         : $Id$
  begin       : Tue May 03 2005
  copyright   : (C) 2005 by Martin Preuss
  email       : martin@libchipcard.de
@@ -41,7 +41,7 @@ int getKeys(AB_BANKING *ab,
   const GWEN_ARGS args[]={
   {
     GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-    GWEN_ArgsTypeChar,            /* type */
+    GWEN_ArgsType_Char,           /* type */
     "bankId",                     /* name */
     0,                            /* minnum */
     1,                            /* maxnum */
@@ -52,7 +52,7 @@ int getKeys(AB_BANKING *ab,
   },
   {
     GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-    GWEN_ArgsTypeChar,            /* type */
+    GWEN_ArgsType_Char,           /* type */
     "customerId",                 /* name */
     0,                            /* minnum */
     1,                            /* maxnum */
@@ -63,7 +63,7 @@ int getKeys(AB_BANKING *ab,
   },
   {
     GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
-    GWEN_ArgsTypeInt,             /* type */
+    GWEN_ArgsType_Int,            /* type */
     "help",                       /* name */
     0,                            /* minnum */
     0,                            /* maxnum */
@@ -87,7 +87,7 @@ int getKeys(AB_BANKING *ab,
     GWEN_BUFFER *ubuf;
 
     ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
-    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutTypeTXT)) {
+    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
       fprintf(stderr, "ERROR: Could not create help string\n");
       return 1;
     }
@@ -102,6 +102,12 @@ int getKeys(AB_BANKING *ab,
     return 2;
   }
 
+  rv=AB_Banking_OnlineInit(ab);
+  if (rv) {
+    DBG_ERROR(0, "Error on init (%d)", rv);
+    return 2;
+  }
+
   pro=AB_Banking_GetProvider(ab, "aqhbci");
   assert(pro);
 
@@ -136,7 +142,7 @@ int getKeys(AB_BANKING *ab,
     AB_IMEXPORTER_CONTEXT *ctx;
 
     ctx=AB_ImExporterContext_new();
-    rv=AH_Provider_GetServerKeys(pro, u, ctx, 0);
+    rv=AH_Provider_GetServerKeys(pro, u, ctx, 0, 0);
     AB_ImExporterContext_free(ctx);
     if (rv) {
       DBG_ERROR(0, "Error getting server keys (%d)", rv);
@@ -145,6 +151,12 @@ int getKeys(AB_BANKING *ab,
     }
   }
 
+  rv=AB_Banking_OnlineFini(ab);
+  if (rv) {
+    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
+    return 5;
+  }
+
   rv=AB_Banking_Fini(ab);
   if (rv) {
     fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
