@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: editctuser.cpp 980 2006-03-24 15:27:55Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -27,6 +27,8 @@
 #include <qcombobox.h>
 #include <qtextbrowser.h>
 #include <qlabel.h>
+#include <qgroupbox.h>
+#include <qcheckbox.h>
 
 #include <qlineedit.h>
 #include <qmessagebox.h>
@@ -40,6 +42,7 @@
 #include <qtextview.h>
 
 #include <gwenhywfar/debug.h>
+#include <gwenhywfar/gui.h>
 
 
 
@@ -53,7 +56,12 @@ EditCtUser::EditCtUser(QBanking *qb,
 ,_app(qb)
 ,_wInfo(wi)
 ,_bankInfo(0)
-,_dataIsOk(false) {
+,_dataIsOk(false)
+,_idCount(0) {
+
+  showSpecialCheck->setChecked(FALSE);
+  specialGroup->hide();
+
   connect(bankCodeButton, SIGNAL(clicked()),
           this, SLOT(slotBankCodeClicked()));
   connect(bankCodeEdit, SIGNAL(lostFocus()),
@@ -62,7 +70,8 @@ EditCtUser::EditCtUser(QBanking *qb,
           this, SLOT(slotBankCodeChanged(const QString&)));
   connect(userCombo, SIGNAL(activated(int)),
           this, SLOT(slotContextActivated(int)));
-
+  connect(showSpecialCheck, SIGNAL(toggled(bool)),
+          this, SLOT(slotSpecialToggled(bool)));
 }
 
 
@@ -73,48 +82,120 @@ EditCtUser::~EditCtUser() {
 
 
 void EditCtUser::init() {
+  int rv;
+  uint32_t idx;
   int i;
-
-  assert(_wInfo->getMedium());
+  bool fromContextCalled=false;
+  uint32_t userFlags;
 
   userCombo->clear();
-  for (i=0; i<5; i++) {
-    GWEN_BUFFER *bufUserId;
-    char numbuf[16];
 
-    bufUserId=GWEN_Buffer_new(0, 64, 0, 1);
-    snprintf(numbuf, sizeof(numbuf), "%d:", i+1);
-    GWEN_Buffer_AppendString(bufUserId, numbuf);
-    if (AH_Medium_ReadContext(_wInfo->getMedium(), i, 0, 0, bufUserId, 0, 0)){
-      GWEN_Buffer_free(bufUserId);
-      break;
-    }
-    userCombo->insertItem(QString::fromUtf8(GWEN_Buffer_GetStart(bufUserId)));
-    GWEN_Buffer_free(bufUserId);
-  }
-
-  DBG_INFO(0, "Using Context %d", _wInfo->getContext());
-
-  userCombo->setCurrentItem(_wInfo->getContext());
-  _fromContext(_wInfo->getContext());
+  bankCodeEdit->setText(QString::fromUtf8(_wInfo->getBankId().c_str()));
+  userIdEdit->setText(QString::fromUtf8(_wInfo->getUserId().c_str()));
+  customerIdEdit->setText(QString::fromUtf8(_wInfo->getCustomerId().c_str()));
+  nameEdit->setText(QString::fromUtf8(_wInfo->getUserName().c_str()));
+  serverEdit->setText(QString::fromUtf8(_wInfo->getServer().c_str()));
+  peerIdEdit->setText(QString::fromUtf8(_wInfo->getPeerId().c_str()));
 
   if (_wInfo->getCryptMode()==AH_CryptMode_Pintan) {
-    i=2;
+    userCombo->setEnabled(false);
+    hbciVersionCombo->setCurrentItem(2);
     hbciVersionCombo->setEnabled(false);
+    if (!(_wInfo->getHttpVersion().empty()))
+      httpVersionCombo->setCurrentText(QString::fromUtf8(_wInfo->getHttpVersion().c_str()));
   }
   else {
+    GWEN_CRYPT_TOKEN *ct;
+    uint32_t idCount;
+
+    httpVersionCombo->setEnabled(false);
+    ct=_wInfo->getToken();
+    assert(ct);
+
+    if (!GWEN_Crypt_Token_IsOpen(ct)) {
+      rv=GWEN_Crypt_Token_Open(ct, 0, 0);
+      if (rv) {
+	DBG_ERROR(0, "Error opening token (%d)", rv);
+	QMessageBox::critical(this,
+			      tr("Error"),
+			      tr("Could not open crypt token"),
+			      QMessageBox::Ok,QMessageBox::NoButton);
+	return;
+      }
+    }
+
+    idCount=32;
+    rv=GWEN_Crypt_Token_GetContextIdList(ct, _idList, &idCount, 0);
+    if (rv) {
+      DBG_ERROR(0, "Error reading context list (%d)", rv);
+      QMessageBox::critical(this,
+			    tr("Error"),
+			    tr("Could not read context list from token"),
+			    QMessageBox::Ok,QMessageBox::NoButton);
+      return;
+    }
+    _idCount=idCount;
+
+    for (idx=0; idx<_idCount; idx++) {
+      QString qs;
+      const char *s;
+      const GWEN_CRYPT_TOKEN_CONTEXT *cctx;
+
+      cctx=GWEN_Crypt_Token_GetContext(ct, _idList[idx], 0);
+      if (cctx) {
+	qs=QString::number(_idList[idx]);
+	qs+=":";
+	s=GWEN_Crypt_Token_Context_GetUserId(cctx);
+	if (s) {
+	  qs+=" ";
+	  qs+=QString::fromUtf8(s);
+	}
+
+	s=GWEN_Crypt_Token_Context_GetServiceId(cctx);
+	if (s) {
+	  qs+="/";
+	  qs+=QString::fromUtf8(s);
+	}
+      }
+      else {
+	qs=tr("Unreadable Context");
+      }
+      userCombo->insertItem(qs);
+      if (_idList[idx]==_wInfo->getContext()) {
+	DBG_INFO(0, "Using Context %d", idx);
+	userCombo->setCurrentItem(idx);
+	_fromContext(idx, false);
+	fromContextCalled=true;
+      }
+    }
+
+    if (!fromContextCalled) {
+      DBG_ERROR(0, "Reading context now");
+      _fromContext(0, false);
+    }
+
     if (_wInfo->getUser()) {
       switch(AH_User_GetHbciVersion(_wInfo->getUser())) {
       case 201: i=0; break;
       case 220: i=2; break;
+      case 300: i=3; break;
       case 210:
       default:  i=1; break;
       }
     }
     else
       i=1;
+    hbciVersionCombo->setCurrentItem(i);
   }
-  hbciVersionCombo->setCurrentItem(i);
+
+  userFlags=_wInfo->getUserFlags();
+  bankSignCheck->setChecked(!(userFlags & AH_USER_FLAGS_BANK_DOESNT_SIGN));
+  bankCounterCheck->setChecked(userFlags & AH_USER_FLAGS_BANK_USES_SIGNSEQ);
+  forceSsl3Check->setChecked(userFlags & AH_USER_FLAGS_FORCE_SSL3);
+  noBase64Check->setChecked(userFlags & AH_USER_FLAGS_NO_BASE64);
+
+  /* validate server address in edit field */
+  _getServerAddr();
 }
 
 
@@ -132,59 +213,6 @@ QString EditCtUser::_getServerAddr() const {
 
 
 
-AH_CRYPT_MODE EditCtUser::_getCryptMode(AH_MEDIUM *m, int idx) {
-  const GWEN_CRYPTTOKEN_CONTEXT *ctx;
-  const GWEN_CRYPTTOKEN_CRYPTINFO *ci;
-  GWEN_CRYPTTOKEN_CRYPTALGO ca;
-  AH_MEDIUM_CTX *mctx;
-  AH_CRYPT_MODE cm;
-  int rv;
-
-  DBG_ERROR(0, "Checking context %d", idx);
-
-  if (!AH_Medium_IsMounted(m)) {
-    rv=AH_Medium_Mount(m);
-    if (rv) {
-      DBG_ERROR(0, "Error mounting (%d)", rv);
-      QMessageBox::critical(this,
-                            tr("Mount Medium"),
-                            tr("Could not mount medium"),
-                            QMessageBox::Ok,QMessageBox::NoButton);
-      return AH_CryptMode_Unknown;
-    }
-  }
-
-  rv=AH_Medium_SelectContext(m, idx);
-  if (rv) {
-    DBG_ERROR(0, "Could not select context %d (%d)", idx, rv);
-    return AH_CryptMode_Unknown;
-  }
-
-  mctx=AH_Medium_GetCurrentContext(m);
-  assert(mctx);
-
-  ctx=AH_MediumCtx_GetTokenContext(mctx);
-  assert(ctx);
-  ci=GWEN_CryptToken_Context_GetCryptInfo(ctx);
-  assert(ci);
-  ca=GWEN_CryptToken_CryptInfo_GetCryptAlgo(ci);
-  if (ca==GWEN_CryptToken_CryptAlgo_RSA)
-    cm=AH_CryptMode_Rdh;
-  else if (ca==GWEN_CryptToken_CryptAlgo_DES_3K)
-    cm=AH_CryptMode_Ddv;
-  else if (ca==GWEN_CryptToken_CryptAlgo_None)
-    cm=AH_CryptMode_Pintan;
-  else {
-    DBG_ERROR(0, "Invalid crypt algo (%s), unable to detect crypt mode",
-              GWEN_CryptToken_CryptAlgo_toString(ca));
-    return AH_CryptMode_Unknown;
-  }
-
-  return cm;
-}
-
-
-
 bool EditCtUser::_checkStringSanity(const char *s) {
   assert(s);
   while(*s) {
@@ -200,11 +228,10 @@ bool EditCtUser::_checkStringSanity(const char *s) {
 
 bool EditCtUser::apply(){
   GWEN_INETADDRESS *addr;
-  GWEN_ERRORCODE err;
+  int err;
   QString qs;
   int i;
   AH_CRYPT_MODE cm;
-  AH_MEDIUM *m;
   AB_USER *u;
   int idx;
   std::string bankId;
@@ -213,16 +240,17 @@ bool EditCtUser::apply(){
   std::string userName;
   std::string fullServerAddr;
   std::string serverAddr;
-  std::string mediumDescr;
+  std::string httpVersion;
+  std::string peerId;
   int hbciVersion;
+  uint32_t userFlags;
 
-  /* do user data page, we have to create the medium etc */
-  m=_wInfo->getMedium();
-  assert(m);
+  /* do user data page */
 
   switch(hbciVersionCombo->currentItem()) {
   case 0:  hbciVersion=201; break;
   case 2:  hbciVersion=220; break;
+  case 3:  hbciVersion=300; break;
   case 1: 
   default: hbciVersion=210; break;
   }
@@ -232,7 +260,7 @@ bool EditCtUser::apply(){
   userId=QBanking::QStringToUtf8String(userIdEdit->text());
   userName=QBanking::QStringToUtf8String(nameEdit->text());
   custId=QBanking::QStringToUtf8String(customerIdEdit->text());
-  mediumDescr=QBanking::QStringToUtf8String(descriptionEdit->text());
+  peerId=QBanking::QStringToUtf8String(peerIdEdit->text());
   qs=_getServerAddr();
   fullServerAddr=QBanking::QStringToUtf8String(qs);
   i=qs.find('/');
@@ -241,6 +269,8 @@ bool EditCtUser::apply(){
   serverAddr=QBanking::QStringToUtf8String(qs);
   idx=userCombo->currentItem();
 
+  httpVersion=QBanking::QStringToUtf8String(httpVersionCombo->currentText());
+
   /* some sanity checks */
   if (!_checkStringSanity(userId.c_str())) {
     QMessageBox::critical(this,
@@ -333,7 +363,6 @@ bool EditCtUser::apply(){
 
   /* get crypt mode */
   cm=_wInfo->getCryptMode();
-  //_getCryptMode(m, _wInfo->getContext());
   if (cm==AH_CryptMode_Unknown) {
     DBG_ERROR(0, "Unknown crypt mode/mount error");
     return false;
@@ -373,31 +402,30 @@ bool EditCtUser::apply(){
   }
 
 
-  // TODO: Select context !!
-
   addr=GWEN_InetAddr_new(GWEN_AddressFamilyIP);
   err=GWEN_InetAddr_SetAddress(addr, serverAddr.c_str());
-  if (!GWEN_Error_IsOk(err)) {
-    GWEN_TYPE_UINT32 wid;
+  if (err) {
+    uint32_t wid;
 
-    wid=_app->showBox(0, QWidget::tr("Please wait"),
-		      QWidget::tr("Resolving host address..."));
+    wid=GWEN_Gui_ProgressLog(0,
+			     GWEN_LoggerLevel_Notice,
+			     QWidget::tr("Resolving host address...").utf8());
     // Note: The QCString::data() is the conversion that will
     // correctly convert the QCString to the char* pointer.
     DBG_INFO(0, "Resolving hostname \"%s\"",
 	     qs.local8Bit().data());
     err=GWEN_InetAddr_SetName(addr, serverAddr.c_str());
     if (wid)
-      AB_Banking_HideBox(_app->getCInterface(), wid);
-    if (!GWEN_Error_IsOk(err)) {
+      GWEN_Gui_HideBox(wid);
+    if (err) {
       DBG_ERROR(0,
-                "Error resolving hostname \"%s\":",
+		"Error resolving hostname \"%s\":",
 		serverAddr.c_str());
       DBG_ERROR_ERR(0, err);
       QMessageBox::critical(this,
 			    QWidget::tr("Network Error"),
 			    QWidget::tr("Could not resolve the server address %1.\n"
-                                        "Maybe there is a typo? Please correct the server address.")
+					"Maybe there is a typo? Please correct the server address.")
 			    .arg(QString::fromUtf8(serverAddr.c_str())),
 			    QMessageBox::Ok,QMessageBox::NoButton);
       GWEN_InetAddr_free(addr);
@@ -406,6 +434,21 @@ bool EditCtUser::apply(){
   }
   GWEN_InetAddr_free(addr);
 
+  /* handle user flags */
+  userFlags=0;
+
+  if (!(bankSignCheck->isChecked()))
+    userFlags|=AH_USER_FLAGS_BANK_DOESNT_SIGN;
+
+  if (bankCounterCheck->isChecked())
+    userFlags|=AH_USER_FLAGS_BANK_USES_SIGNSEQ;
+
+  if (forceSsl3Check->isChecked())
+    userFlags|=AH_USER_FLAGS_FORCE_SSL3;
+
+  if (noBase64Check->isChecked())
+    userFlags|=AH_USER_FLAGS_NO_BASE64;
+  _wInfo->setUserFlags(userFlags);
 
   /* select or create user */
   u=AB_Banking_FindUser(_app->getCInterface(),
@@ -425,10 +468,29 @@ bool EditCtUser::apply(){
     AB_User_SetUserId(u, userId.c_str());
     AB_User_SetCustomerId(u, custId.c_str());
     AB_User_SetUserName(u, userName.c_str());
-    AH_User_SetMedium(u, m);
+    if (!peerId.empty())
+      AH_User_SetPeerId(u, peerId.c_str());
+
+    AH_User_SetTokenType(u, _wInfo->getMediumType().c_str());
+    AH_User_SetTokenName(u, _wInfo->getMediumName().c_str());
+    _wInfo->setContext(_idList[idx]);
+    AH_User_SetTokenContextId(u, _wInfo->getContext());
+
     AH_User_SetCryptMode(u, cm);
     AH_User_SetHbciVersion(u, hbciVersion);
 
+    AH_User_AddFlags(u, userFlags);
+
+    if (!(httpVersion.empty())) {
+      int vmajor, vminor;
+
+      if (sscanf(httpVersion.c_str(), "%d.%d", &vmajor, &vminor)==2) {
+        DBG_ERROR(0, "Setting HTTP_Version to %d.%d", vmajor, vminor);
+        AH_User_SetHttpVMajor(u, vmajor);
+        AH_User_SetHttpVMinor(u, vminor);
+      }
+    }
+
     /* set address */
     url=GWEN_Url_fromString(fullServerAddr.c_str());
     assert(url);
@@ -443,6 +505,7 @@ bool EditCtUser::apply(){
     AH_User_SetServerUrl(u, url);
     GWEN_Url_free(url);
 
+    AB_Banking_AddUser(_app->getCInterface(), u);
     _wInfo->setUser(u);
     _wInfo->addFlags(WIZARDINFO_FLAGS_USER_CREATED);
   }
@@ -469,13 +532,6 @@ bool EditCtUser::apply(){
     return false;
   }
 
-  _wInfo->setContext(idx);
-  AH_User_SetContextIdx(u, idx);
-
-  /* Set a descriptiveName in the medium. */
-  if (!mediumDescr.empty())
-    AH_Medium_SetDescriptiveName(m, mediumDescr.c_str());
-
   if (AH_User_GetCryptMode(u)==AH_CryptMode_Pintan) {
     /* PIN/TAN only works with HBCI version 2.20. It might work with
      * other versions as well but this is not defined in the specs. */
@@ -494,9 +550,9 @@ bool EditCtUser::undo(){
   if (u) {
     if (_wInfo->getFlags() & WIZARDINFO_FLAGS_USER_CREATED) {
       DBG_INFO(0, "Removing user and all subordinate objects");
+      AB_Banking_DeleteUser(_app->getCInterface(), u);
       _wInfo->setUser(0);
       _wInfo->subFlags(WIZARDINFO_FLAGS_USER_CREATED);
-      AB_User_free(u);
     } // if _userCreated
   } // if user
 
@@ -531,11 +587,23 @@ void EditCtUser::slotBankCodeLostFocus() {
 
 
 void EditCtUser::slotBankCodeChanged(const QString&) {
+  std::string s;
+
   if (_bankInfo) {
     DBG_ERROR(0, "Deleting current bank info");
     AB_BankInfo_free(_bankInfo);
     _bankInfo=0;
   }
+
+  s=QBanking::QStringToUtf8String(bankCodeEdit->text());
+  if (!s.empty()) {
+    if (strcasecmp(s.c_str(), "30060601")==0) {
+      /* special handling of DT. Apotheker and Aerztebank: use
+       * NO_BASE64 */
+      noBase64Check->setChecked(true);
+      _wInfo->addUserFlags(AH_USER_FLAGS_NO_BASE64);
+    }
+  }
 }
 
 
@@ -547,8 +615,9 @@ void EditCtUser::slotBankCodeClicked() {
   _bankInfo=0;
   bi=QBSelectBank::selectBank(_app,
                               0,
-                              tr("Select a bank"),
-                              bankCodeEdit->text());
+			      tr("Select a bank"),
+			      "de",
+			      bankCodeEdit->text());
   if (bi) {
     const char *s;
     AB_BANKINFO_SERVICE *sv;
@@ -559,8 +628,7 @@ void EditCtUser::slotBankCodeClicked() {
       bankCodeEdit->setText(QString::fromUtf8(s));
 
     sv=AB_BankInfoService_List_First(AB_BankInfo_GetServices(bi));
-    isPinTan=_getCryptMode(_wInfo->getMedium(), _wInfo->getContext())==
-      AH_CryptMode_Pintan;
+    isPinTan=(_wInfo->getCryptMode()==AH_CryptMode_Pintan);
 
     while(sv) {
       s=AB_BankInfoService_GetType(sv);
@@ -582,48 +650,89 @@ void EditCtUser::slotBankCodeClicked() {
 
 
 
-void EditCtUser::_fromContext(int i) {
-  int country;
-  GWEN_BUFFER *bankId;
-  GWEN_BUFFER *userId;
-  GWEN_BUFFER *server;
-  int port;
-  int rv;
+void EditCtUser::_fromContext(int i, bool overwrite) {
+  if (i<(int)_idCount) {
+    GWEN_CRYPT_TOKEN *ct;
+    const GWEN_CRYPT_TOKEN_CONTEXT *cctx;
+    int rv;
 
-  bankId=GWEN_Buffer_new(0, 64, 0, 1);
-  userId=GWEN_Buffer_new(0, 64, 0, 1);
-  server=GWEN_Buffer_new(0, 64, 0, 1);
-  rv=AH_Medium_ReadContext(_wInfo->getMedium(),
-                           i, &country, bankId, userId, server, &port);
-  if (rv) {
-    DBG_ERROR(0, "Could not read context %d", i);
-    GWEN_Buffer_free(server);
-    GWEN_Buffer_free(userId);
-    GWEN_Buffer_free(bankId);
-    return;
+    ct=_wInfo->getToken();
+    assert(ct);
+
+    if (!GWEN_Crypt_Token_IsOpen(ct)) {
+      rv=GWEN_Crypt_Token_Open(ct, 0, 0);
+      if (rv) {
+	DBG_ERROR(0, "Error opening token (%d)", rv);
+	QMessageBox::critical(this,
+			      tr("Error"),
+			      tr("Could not open crypt token"),
+			      QMessageBox::Ok,QMessageBox::NoButton);
+	return;
+      }
+    }
+
+    if (userIdEdit->text().isEmpty()) {
+      DBG_ERROR(0, "User Id is empty");
+    }
+    else {
+      DBG_ERROR(0, "User Id is not empty");
+    }
+
+    cctx=GWEN_Crypt_Token_GetContext(ct, _idList[i], 0);
+    if (cctx) {
+      const char *s;
+
+      s=GWEN_Crypt_Token_Context_GetUserId(cctx);
+      if (s) {
+        DBG_ERROR(0, "User id available");
+	if (overwrite || userIdEdit->text().isEmpty())
+	  userIdEdit->setText(QString::fromUtf8(s));
+	if (overwrite || customerIdEdit->text().isEmpty())
+	  customerIdEdit->setText(QString::fromUtf8(s));
+      }
+      else {
+	DBG_ERROR(0, "User id not available");
+      }
+      s=GWEN_Crypt_Token_Context_GetServiceId(cctx);
+      if (s && (overwrite || bankCodeEdit->text().isEmpty()))
+	bankCodeEdit->setText(QString::fromUtf8(s));
+      s=GWEN_Crypt_Token_Context_GetAddress(cctx);
+      if (s && (overwrite || serverEdit->text().isEmpty()))
+	serverEdit->setText(QString::fromUtf8(s));
+      s=GWEN_Crypt_Token_Context_GetUserName(cctx);
+      if (s && (overwrite || nameEdit->text().isEmpty()))
+	nameEdit->setText(QString::fromUtf8(s));
+      s=GWEN_Crypt_Token_Context_GetPeerId(cctx);
+      if (overwrite || peerIdEdit->text().isEmpty())
+	peerIdEdit->setText(QString::fromUtf8(s));
+    }
+    _wInfo->setContext(_idList[i]);
+    DBG_ERROR(0, "Using context %d", i);
+  }
+  else {
+    DBG_ERROR(0, "Invalid context %d", i);
   }
-
-  bankCodeEdit->setText(QString::fromUtf8(GWEN_Buffer_GetStart(bankId)));
-  userIdEdit->setText(QString::fromUtf8(GWEN_Buffer_GetStart(userId)));
-  customerIdEdit->setText("");
-  serverEdit->setText(QString::fromUtf8(GWEN_Buffer_GetStart(server)));
-  GWEN_Buffer_free(server);
-  GWEN_Buffer_free(userId);
-  GWEN_Buffer_free(bankId);
 }
 
 
 
 void EditCtUser::slotContextActivated(int i) {
-  _wInfo->setContext(i);
   _fromContext(i);
 }
 
 
 
+void EditCtUser::slotSpecialToggled(bool on) {
+  if (on)
+    specialGroup->show();
+  else
+    specialGroup->hide();
+}
 
 
 
+#include "editctuser.moc"
+
 
 
 
