@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: winfo.cpp 950 2006-02-22 19:39:02Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -18,6 +18,8 @@
 #include "winfo.h"
 #include <aqhbci/provider.h>
 
+#include <aqbanking/banking_be.h>
+
 #include <gwenhywfar/debug.h>
 #include <unistd.h>
 
@@ -27,12 +29,13 @@
 WizardInfo::WizardInfo(AB_PROVIDER *pro)
 :_provider(pro)
 ,_user(0)
-,_medium(0)
+,_token(0)
 ,_context(0)
 ,_country(280)
 ,_cryptMode(AH_CryptMode_None)
 ,_port(3000)
-,_flags(0) {
+,_flags(0)
+,_userFlags(0) {
 
 
 }
@@ -50,22 +53,19 @@ AB_PROVIDER *WizardInfo::getProvider() const {
 
 
 
-AH_MEDIUM *WizardInfo::getMedium() const {
-  return _medium;
+GWEN_CRYPT_TOKEN *WizardInfo::getToken() const {
+  return _token;
 }
 
 
 
-void WizardInfo::setMedium(AH_MEDIUM *m) {
-  if (_medium) {
-    if (m) {
-      DBG_ERROR(0, "Overwriting existing medium!")
-    }
-    else {
-      DBG_ERROR(0, "Resetting medium")
+void WizardInfo::setToken(GWEN_CRYPT_TOKEN *ct) {
+  if (_token) {
+    if (ct) {
+      DBG_ERROR(0, "Overwriting existing token!")
     }
   }
-  _medium=m;
+  _token=ct;
 }
 
 
@@ -82,13 +82,13 @@ void WizardInfo::setCryptMode(AH_CRYPT_MODE cm) {
 
 
 
-int WizardInfo::getContext() const {
+uint32_t WizardInfo::getContext() const {
   return _context;
 }
 
 
 
-void WizardInfo::setContext(int i) {
+void WizardInfo::setContext(uint32_t i) {
   _context=i;
 }
 
@@ -166,6 +166,18 @@ void WizardInfo::setCustomerId(const std::string &s) {
 
 
 
+const std::string &WizardInfo::getPeerId() const {
+  return _peerId;
+}
+
+
+
+void WizardInfo::setPeerId(const std::string &s) {
+  _peerId=s;
+}
+
+
+
 const std::string &WizardInfo::getServer() const {
   return _server;
 }
@@ -190,6 +202,30 @@ void WizardInfo::setPort(int i) {
 
 
 
+const std::string &WizardInfo::getHttpVersion() const {
+  return _httpVersion;
+}
+
+
+
+void WizardInfo::setHttpVersion(const std::string &s) {
+  _httpVersion=s;
+}
+
+
+
+const std::string &WizardInfo::getMediumType() const {
+  return _mediumType;
+}
+
+
+
+void WizardInfo::setMediumType(const std::string &s) {
+  _mediumType=s;
+}
+
+
+
 const std::string &WizardInfo::getMediumName() const {
   return _mediumName;
 }
@@ -202,53 +238,73 @@ void WizardInfo::setMediumName(const std::string &s) {
 
 
 
-GWEN_TYPE_UINT32 WizardInfo::getFlags() const {
+uint32_t WizardInfo::getFlags() const {
   return _flags;
 }
 
 
 
-void WizardInfo::setFlags(GWEN_TYPE_UINT32 fl) {
+void WizardInfo::setFlags(uint32_t fl) {
   _flags=fl;
 }
 
 
 
-void WizardInfo::addFlags(GWEN_TYPE_UINT32 fl) {
+void WizardInfo::addFlags(uint32_t fl) {
   _flags|=fl;
 }
 
 
 
-void WizardInfo::subFlags(GWEN_TYPE_UINT32 fl) {
+void WizardInfo::subFlags(uint32_t fl) {
   _flags&=~fl;
 }
 
 
 
+uint32_t WizardInfo::getUserFlags() const {
+  return _userFlags;
+}
+
+
+
+void WizardInfo::setUserFlags(uint32_t fl) {
+  _userFlags=fl;
+}
+
+
+
+void WizardInfo::addUserFlags(uint32_t fl) {
+  _userFlags|=fl;
+}
+
+
+
+void WizardInfo::subUserFlags(uint32_t fl) {
+  _userFlags&=~fl;
+}
+
+
+
 void WizardInfo::releaseData() {
   // handle user
   if (_user) {
+
     if (_flags & WIZARDINFO_FLAGS_USER_CREATED) {
       /* user created, so by removing the user we also remove all other
        * objects below it */
+      AB_Banking_DeleteUser(AB_Provider_GetBanking(_provider), _user);
       DBG_INFO(0, "Removing user and all subordinate objects");
       _flags&=~WIZARDINFO_FLAGS_USER_CREATED;
-      AB_User_free(_user);
       _user=0;
     } // if _userCreated
   } // if user
 
-  if (_medium && (_flags & WIZARDINFO_FLAGS_MEDIUM_CREATED)) {
-    if (_flags & WIZARDINFO_FLAGS_MEDIUM_ADDED) {
-      DBG_INFO(0, "Unlisting medium");
-      AH_Provider_RemoveMedium(_provider, _medium);
-      _flags&=~WIZARDINFO_FLAGS_MEDIUM_ADDED;
-    }
+  if (_token && (_flags & WIZARDINFO_FLAGS_MEDIUM_CREATED)) {
     DBG_INFO(0, "Deleting medium");
-    AH_Medium_free(_medium);
+    AB_Banking_ClearCryptTokenList(AB_Provider_GetBanking(_provider), 0);
     _flags&=~WIZARDINFO_FLAGS_MEDIUM_CREATED;
-    _medium=0;
+    _token=NULL;
   }
 
   if (!_mediumName.empty() &&
