@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: provider.c 865 2006-01-27 03:04:21Z aquamaniac $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -48,14 +48,14 @@ int AN_Provider_Fini(AB_PROVIDER *pro, GWEN_DB_NODE *dbData){
 
 
 
-int AN_Provider_UpdateJob(AB_PROVIDER *pro, AB_JOB *j){
-  return AB_ERROR_NOT_SUPPORTED;
+int AN_Provider_UpdateJob(AB_PROVIDER *pro, AB_JOB *j, uint32_t guiid){
+  return GWEN_ERROR_NOT_SUPPORTED;
 }
 
 
 
-int AN_Provider_AddJob(AB_PROVIDER *pro, AB_JOB *j){
-  return AB_ERROR_NOT_SUPPORTED;
+int AN_Provider_AddJob(AB_PROVIDER *pro, AB_JOB *j, uint32_t guiid){
+  return GWEN_ERROR_NOT_SUPPORTED;
 }
 
 
@@ -66,7 +66,8 @@ int AN_Provider_ResetQueue(AB_PROVIDER *pro){
 
 
 
-int AN_Provider_Execute(AB_PROVIDER *pro, AB_IMEXPORTER_CONTEXT *ctx){
+int AN_Provider_Execute(AB_PROVIDER *pro, AB_IMEXPORTER_CONTEXT *ctx,
+			uint32_t guiid){
   return 0;
 }
 
