@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: account_p.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: account_p.h,v 1.3 2006/01/19 01:07:48 aquamaniac Exp $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
