@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: user_p.h 804 2006-01-19 01:07:49Z aquamaniac $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -18,10 +18,10 @@
 
 typedef struct AO_USER AO_USER;
 struct AO_USER {
-  GWEN_TYPE_UINT32 flags;
+  uint32_t flags;
 };
 
-static void AO_User_FreeData(void *bp, void *p);
+static void GWENHYWFAR_CB AO_User_FreeData(void *bp, void *p);
 
 
 
