@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: version.h.in 418 2005-08-08 02:24:41Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -14,13 +14,13 @@
 #ifndef AQOFXCONNECT_VERSION_H
 #define AQOFXCONNECT_VERSION_H
 
-#define AQOFXCONNECT_VERSION_MAJOR 2
-#define AQOFXCONNECT_VERSION_MINOR 2
-#define AQOFXCONNECT_VERSION_PATCHLEVEL 3
+#define AQOFXCONNECT_VERSION_MAJOR 3
+#define AQOFXCONNECT_VERSION_MINOR 5
+#define AQOFXCONNECT_VERSION_PATCHLEVEL 1
 #define AQOFXCONNECT_VERSION_BUILD 0
 #define AQOFXCONNECT_VERSION_TAG "stable"
-#define AQOFXCONNECT_VERSION_FULL_STRING "2.2.3.0-stable"
-#define AQOFXCONNECT_VERSION_STRING "2.2.3"
+#define AQOFXCONNECT_VERSION_FULL_STRING "3.5.1.0stable"
+#define AQOFXCONNECT_VERSION_STRING "3.5.1"
 
 
 
