@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: csv.c 1038 2006-07-03 03:01:30Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -16,15 +16,35 @@
 
 #include "csv_p.h"
 #include "i18n_l.h"
+
+#include "imexporter_be.h"
+
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/text.h>
-#include <gwenhywfar/waitcallback.h>
+#include <gwenhywfar/gui.h>
 
 
 GWEN_INHERIT(AB_IMEXPORTER, AH_IMEXPORTER_CSV);
 
 
-AB_IMEXPORTER *csv_factory(AB_BANKING *ab, GWEN_DB_NODE *db){
+GWEN_PLUGIN *imexporters_csv_factory(GWEN_PLUGIN_MANAGER *pm,
+				     const char *name,
+				     const char *fileName) {
+  GWEN_PLUGIN *pl;
+
+  pl=AB_Plugin_ImExporter_new(pm, name, fileName);
+  assert(pl);
+
+  AB_Plugin_ImExporter_SetFactoryFn(pl, AB_Plugin_ImExporterCSV_Factory);
+
+  return pl;
+}
+
+
+
+AB_IMEXPORTER *AB_Plugin_ImExporterCSV_Factory(GWEN_PLUGIN *pl,
+					       AB_BANKING *ab,
+					       GWEN_DB_NODE *db){
   AB_IMEXPORTER *ie;
   AH_IMEXPORTER_CSV *ieh;
 
@@ -49,7 +69,7 @@ AB_IMEXPORTER *csv_factory(AB_BANKING *ab, GWEN_DB_NODE *db){
 
 
 
-void AH_ImExporterCSV_FreeData(void *bp, void *p){
+void GWENHYWFAR_CB AH_ImExporterCSV_FreeData(void *bp, void *p){
   AH_IMEXPORTER_CSV *ieh;
 
   ieh=(AH_IMEXPORTER_CSV*)p;
@@ -59,9 +79,10 @@ void AH_ImExporterCSV_FreeData(void *bp, void *p){
 
 
 int AH_ImExporterCSV_Import(AB_IMEXPORTER *ie,
-                            AB_IMEXPORTER_CONTEXT *ctx,
-                            GWEN_BUFFEREDIO *bio,
-                            GWEN_DB_NODE *params){
+			    AB_IMEXPORTER_CONTEXT *ctx,
+			    GWEN_IO_LAYER *io,
+			    GWEN_DB_NODE *params,
+			    uint32_t guiid){
   AH_IMEXPORTER_CSV *ieh;
   GWEN_DB_NODE *dbData;
   GWEN_DB_NODE *dbSubParams;
@@ -76,34 +97,36 @@ int AH_ImExporterCSV_Import(AB_IMEXPORTER *ie,
 			       "params");
   dbData=GWEN_DB_Group_new("transactions");
   rv=GWEN_DBIO_Import(ieh->dbio,
-                      bio,
-                      GWEN_DB_FLAGS_DEFAULT |
-                      GWEN_PATH_FLAGS_CREATE_GROUP,
+		      io,
                       dbData,
-		      dbSubParams);
+		      dbSubParams,
+		      GWEN_DB_FLAGS_DEFAULT |
+		      GWEN_PATH_FLAGS_CREATE_GROUP,
+		      guiid,
+		      2000);
   if (rv) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Error importing data (%d)", rv);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
-                          "Error importing data");
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
+			 "Error importing data");
     GWEN_DB_Group_free(dbData);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
 
   /* transform DB to transactions */
-  GWEN_WaitCallback_Log(GWEN_LoggerLevelNotice,
-                        I18N("Data imported, transforming to UTF-8"));
+  GWEN_Gui_ProgressLog(guiid, GWEN_LoggerLevel_Notice,
+		       I18N("Data imported, transforming to UTF-8"));
   rv=AB_ImExporter_DbFromIso8859_1ToUtf8(dbData);
   if (rv) {
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(guiid, GWEN_LoggerLevel_Error,
                           "Error converting data");
     GWEN_DB_Group_free(dbData);
     return rv;
   }
-  GWEN_WaitCallback_Log(GWEN_LoggerLevelNotice,
-                        "Transforming data to transactions");
-  rv=AH_ImExporterCSV__ImportFromGroup(ctx, dbData, params);
+  GWEN_Gui_ProgressLog(guiid, GWEN_LoggerLevel_Notice,
+		       "Transforming data to transactions");
+  rv=AH_ImExporterCSV__ImportFromGroup(ctx, dbData, params, guiid);
   if (rv) {
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(guiid, GWEN_LoggerLevel_Error,
                           "Error importing data");
     GWEN_DB_Group_free(dbData);
     return rv;
@@ -117,30 +140,15 @@ int AH_ImExporterCSV_Import(AB_IMEXPORTER *ie,
 
 int AH_ImExporterCSV__ImportFromGroup(AB_IMEXPORTER_CONTEXT *ctx,
                                       GWEN_DB_NODE *db,
-                                      GWEN_DB_NODE *dbParams) {
+				      GWEN_DB_NODE *dbParams,
+				      uint32_t guiid) {
   GWEN_DB_NODE *dbT;
   const char *dateFormat;
   int inUtc;
   int usePosNegField;
   int defaultIsPositive;
   const char *posNegFieldName;
-  GWEN_TYPE_UINT64 cnt=0;
-  GWEN_TYPE_UINT64 done;
-
-  /* first count the groups */
-  dbT=GWEN_DB_GetFirstGroup(db);
-  while(dbT) {
-    cnt++;
-    dbT=GWEN_DB_GetNextGroup(dbT);
-  } /* while */
-
-  /* enter waitcallback context */
-  GWEN_WaitCallback_EnterWithText(GWEN_WAITCALLBACK_ID_SIMPLE_PROGRESS,
-                                  I18N("Importing transactions..."),
-                                  I18N("transaction(s)"),
-                                  GWEN_WAITCALLBACK_FLAGS_NO_REUSE);
-  GWEN_WaitCallback_SetProgressTotal(cnt);
-  GWEN_WaitCallback_SetProgressPos(0);
+  uint32_t progressId;
 
   dateFormat=GWEN_DB_GetCharValue(dbParams, "dateFormat", 0, "YYYY/MM/DD");
   inUtc=GWEN_DB_GetIntValue(dbParams, "utc", 0, 0);
@@ -149,7 +157,14 @@ int AH_ImExporterCSV__ImportFromGroup(AB_IMEXPORTER_CONTEXT *ctx,
   posNegFieldName=GWEN_DB_GetCharValue(dbParams, "posNegFieldName", 0,
 				       "posNeg");
 
-  done=0;
+  progressId=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+				    GWEN_GUI_PROGRESS_ALLOW_EMBED |
+				    GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+				    GWEN_GUI_PROGRESS_SHOW_ABORT,
+				    I18N("Importing parsed data..."),
+				    NULL,
+				    GWEN_DB_Groups_Count(db),
+				    guiid);
   dbT=GWEN_DB_GetFirstGroup(db);
   while(dbT) {
     int matches;
@@ -187,10 +202,10 @@ int AH_ImExporterCSV__ImportFromGroup(AB_IMEXPORTER_CONTEXT *ctx,
         t=AB_Transaction_fromDb(dbT);
         if (!t) {
           DBG_ERROR(AQBANKING_LOGDOMAIN, "Error in config file");
-          GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
-                                "Error in config file");
-          GWEN_WaitCallback_Leave();
-          return AB_ERROR_GENERIC;
+          GWEN_Gui_ProgressLog(progressId, GWEN_LoggerLevel_Error,
+			       "Error in config file");
+	  GWEN_Gui_ProgressEnd(progressId);
+          return GWEN_ERROR_GENERIC;
         }
   
         /* translate date */
@@ -296,30 +311,33 @@ int AH_ImExporterCSV__ImportFromGroup(AB_IMEXPORTER_CONTEXT *ctx,
 
       DBG_INFO(AQBANKING_LOGDOMAIN, "Not a transaction, checking subgroups");
       /* not a transaction, check subgroups */
-      rv=AH_ImExporterCSV__ImportFromGroup(ctx, dbT, dbParams);
+      rv=AH_ImExporterCSV__ImportFromGroup(ctx, dbT, dbParams, guiid);
       if (rv) {
         DBG_INFO(AQBANKING_LOGDOMAIN, "here");
-        GWEN_WaitCallback_Leave();
+	GWEN_Gui_ProgressEnd(progressId);
         return rv;
       }
     }
-    done++;
-    if (GWEN_WaitCallbackProgress(done)==GWEN_WaitCallbackResult_Abort) {
-      GWEN_WaitCallback_Leave();
-      return AB_ERROR_USER_ABORT;
+
+    if (GWEN_Gui_ProgressAdvance(progressId, GWEN_GUI_PROGRESS_ONE)==
+	GWEN_ERROR_USER_ABORTED) {
+      GWEN_Gui_ProgressLog(progressId, GWEN_LoggerLevel_Error,
+			   I18N("Aborted by user"));
+      GWEN_Gui_ProgressEnd(progressId);
+      return GWEN_ERROR_USER_ABORTED;
     }
-    GWEN_WaitCallback_SetProgressPos(done);
+
     dbT=GWEN_DB_GetNextGroup(dbT);
   } // while
 
-  GWEN_WaitCallback_Leave();
+  GWEN_Gui_ProgressEnd(progressId);
 
   return 0;
 }
 
 
 
-int AH_ImExporterCSV_CheckFile(AB_IMEXPORTER *ie, const char *fname){
+int AH_ImExporterCSV_CheckFile(AB_IMEXPORTER *ie, const char *fname, uint32_t guiid){
   AH_IMEXPORTER_CSV *ieh;
   GWEN_DBIO_CHECKFILE_RESULT rv;
 
@@ -328,21 +346,22 @@ int AH_ImExporterCSV_CheckFile(AB_IMEXPORTER *ie, const char *fname){
   assert(ieh);
   assert(ieh->dbio);
 
-  rv=GWEN_DBIO_CheckFile(ieh->dbio, fname);
+  rv=GWEN_DBIO_CheckFile(ieh->dbio, fname, guiid, 2000);
   switch(rv) {
   case GWEN_DBIO_CheckFileResultOk:      return 0;
-  case GWEN_DBIO_CheckFileResultNotOk:   return AB_ERROR_BAD_DATA;
-  case GWEN_DBIO_CheckFileResultUnknown: return AB_ERROR_UNKNOWN;
-  default:                               return AB_ERROR_GENERIC;
+  case GWEN_DBIO_CheckFileResultNotOk:   return GWEN_ERROR_BAD_DATA;
+  case GWEN_DBIO_CheckFileResultUnknown: return AB_ERROR_INDIFFERENT;
+  default:                               return GWEN_ERROR_GENERIC;
   } /* switch */
 }
 
 
 
-int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
-                            AB_IMEXPORTER_CONTEXT *ctx,
-                            GWEN_BUFFEREDIO *bio,
-                            GWEN_DB_NODE *params){
+int AH_ImExporterCSV__ExportTransactions(AB_IMEXPORTER *ie,
+					 AB_IMEXPORTER_CONTEXT *ctx,
+					 GWEN_IO_LAYER *io,
+					 GWEN_DB_NODE *params,
+					 uint32_t guiid){
   AH_IMEXPORTER_CSV *ieh;
   AB_IMEXPORTER_ACCOUNTINFO *ai;
   GWEN_DB_NODE *dbData;
@@ -353,6 +372,7 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
   int usePosNegField;
   int defaultIsPositive;
   const char *posNegFieldName;
+  const char *valueFormat;
 
   assert(ie);
   ieh=GWEN_INHERIT_GETDATA(AB_IMEXPORTER, AH_IMEXPORTER_CSV, ie);
@@ -368,6 +388,8 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
   defaultIsPositive=GWEN_DB_GetIntValue(params, "defaultIsPositive", 0, 1);
   posNegFieldName=GWEN_DB_GetCharValue(params, "posNegFieldName", 0,
 				       "posNeg");
+  valueFormat=GWEN_DB_GetCharValue(params, "valueFormat", 0, "float");
+
 
   /* create db, store transactions in it */
   dbData=GWEN_DB_Group_new("transactions");
@@ -379,37 +401,17 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
     while(t) {
       GWEN_DB_NODE *dbTransaction;
       const GWEN_TIME *ti;
-      AB_SPLIT *sp;
 
       dbTransaction=GWEN_DB_Group_new("transaction");
       rv=AB_Transaction_toDb(t, dbTransaction);
       if (rv) {
         DBG_ERROR(AQBANKING_LOGDOMAIN,
                   "Could not transform transaction to db");
-        GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
-                              "Error transforming data to db");
+        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
+			     "Error transforming data to db");
         GWEN_DB_Group_free(dbData);
         GWEN_DB_Group_free(dbTransaction);
-        return AB_ERROR_GENERIC;
-      }
-
-      /* if transaction contains splits:
-       * Take the first split and store it within the transaction thus
-       * overwriting possibly existing data in the transaction itself.
-       * However, if there actually ARE splits then the corresponding data
-       * within the transaction is NULL anyway so this step doesn't hurt. */
-      sp=AB_Split_List_First(AB_Transaction_GetSplits(t));
-      if (sp) {
-        rv=AB_Split_toDb(sp, dbTransaction);
-        if (rv) {
-          DBG_ERROR(AQBANKING_LOGDOMAIN,
-                    "Could not transform split to db");
-          GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
-                                "Error transforming data to db");
-          GWEN_DB_Group_free(dbData);
-          GWEN_DB_Group_free(dbTransaction);
-          return AB_ERROR_GENERIC;
-        }
+        return GWEN_ERROR_GENERIC;
       }
 
       /* transform dates */
@@ -472,7 +474,7 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
 	      DBG_ERROR(AQBANKING_LOGDOMAIN,
                         "No value for \"positiveValues\" in params");
 	      GWEN_DB_Group_free(dbData);
-	      return AB_ERROR_GENERIC;
+	      return GWEN_ERROR_GENERIC;
 	    }
 	  }
 	  else {
@@ -495,19 +497,37 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
 		DBG_ERROR(AQBANKING_LOGDOMAIN,
 			  "Could not store value to DB");
 		GWEN_DB_Group_free(dbData);
-		return AB_ERROR_GENERIC;
+		return GWEN_ERROR_GENERIC;
 	      }
 	    }
 	    else {
 	      DBG_ERROR(AQBANKING_LOGDOMAIN,
 			"No value for \"negativeValues\" in params");
 	      GWEN_DB_Group_free(dbData);
-	      return AB_ERROR_GENERIC;
+	      return GWEN_ERROR_GENERIC;
 	    }
 	  }
 	}
       }
 
+      if (strcasecmp(valueFormat, "float")==0) {
+	GWEN_DB_NODE *dbV;
+
+	dbV=GWEN_DB_GetGroup(dbTransaction,
+			     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+			     "value");
+	if (dbV) {
+	  AB_VALUE *v;
+
+	  v=AB_Value_fromDb(dbV);
+	  if (v) {
+	    GWEN_DB_ClearGroup(dbV, 0);
+	    AB_Value_toDbFloat(v, dbV);
+            AB_Value_free(v);
+	  }
+	}
+      }
+
       /* add transaction db */
       GWEN_DB_AddGroup(dbData, dbTransaction);
       t=AB_ImExporterAccountInfo_GetNextTransaction(ai);
@@ -517,16 +537,18 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
 
 
   rv=GWEN_DBIO_Export(ieh->dbio,
-                      bio,
-                      GWEN_DB_FLAGS_DEFAULT,
-                      dbData,
-                      dbSubParams);
+		      io,
+		      dbData,
+		      dbSubParams,
+		      GWEN_DB_FLAGS_DEFAULT,
+		      guiid,
+                      2000);
   if (rv) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Error exporting data (%d)", rv);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
-                          "Error exporting data");
+    GWEN_Gui_ProgressLog(guiid, GWEN_LoggerLevel_Error,
+			 "Error exporting data");
     GWEN_DB_Group_free(dbData);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
   GWEN_DB_Group_free(dbData);
 
@@ -535,5 +557,212 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
 
 
 
+int AH_ImExporterCSV__ExportNotedTransactions(AB_IMEXPORTER *ie,
+					      AB_IMEXPORTER_CONTEXT *ctx,
+					      GWEN_IO_LAYER *io,
+					      GWEN_DB_NODE *params,
+					      uint32_t guiid){
+  AH_IMEXPORTER_CSV *ieh;
+  AB_IMEXPORTER_ACCOUNTINFO *ai;
+  GWEN_DB_NODE *dbData;
+  GWEN_DB_NODE *dbSubParams;
+  int rv;
+  const char *dateFormat;
+  int inUtc;
+  int usePosNegField;
+  int defaultIsPositive;
+  const char *posNegFieldName;
+
+  assert(ie);
+  ieh=GWEN_INHERIT_GETDATA(AB_IMEXPORTER, AH_IMEXPORTER_CSV, ie);
+  assert(ieh);
+  assert(ieh->dbio);
+
+  dbSubParams=GWEN_DB_GetGroup(params, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+                               "params");
+  dateFormat=GWEN_DB_GetCharValue(params, "dateFormat", 0,
+				  "YYYY/MM/DD");
+  inUtc=GWEN_DB_GetIntValue(params, "utc", 0, 0);
+  usePosNegField=GWEN_DB_GetIntValue(params, "usePosNegField", 0, 0);
+  defaultIsPositive=GWEN_DB_GetIntValue(params, "defaultIsPositive", 0, 1);
+  posNegFieldName=GWEN_DB_GetCharValue(params, "posNegFieldName", 0,
+				       "posNeg");
+
+  /* create db, store transactions in it */
+  dbData=GWEN_DB_Group_new("transactions");
+  ai=AB_ImExporterContext_GetFirstAccountInfo(ctx);
+  while(ai) {
+    const AB_TRANSACTION *t;
+
+    t=AB_ImExporterAccountInfo_GetFirstNotedTransaction(ai);
+    while(t) {
+      GWEN_DB_NODE *dbTransaction;
+      const GWEN_TIME *ti;
+
+      dbTransaction=GWEN_DB_Group_new("transaction");
+      rv=AB_Transaction_toDb(t, dbTransaction);
+      if (rv) {
+        DBG_ERROR(AQBANKING_LOGDOMAIN,
+                  "Could not transform transaction to db");
+        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
+			     "Error transforming data to db");
+        GWEN_DB_Group_free(dbData);
+        GWEN_DB_Group_free(dbTransaction);
+        return GWEN_ERROR_GENERIC;
+      }
+
+      /* transform dates */
+      GWEN_DB_DeleteGroup(dbTransaction, "date");
+      GWEN_DB_DeleteGroup(dbTransaction, "valutaDate");
+
+      ti=AB_Transaction_GetDate(t);
+      if (ti) {
+	GWEN_BUFFER *tbuf;
+        int rv;
+
+	tbuf=GWEN_Buffer_new(0, 32, 0, 1);
+	if (inUtc)
+	  rv=GWEN_Time_toUtcString(ti, dateFormat, tbuf);
+	else
+	  rv=GWEN_Time_toString(ti, dateFormat, tbuf);
+	if (rv) {
+	  DBG_WARN(AQBANKING_LOGDOMAIN, "Bad date format string/date");
+	}
+	else
+	  GWEN_DB_SetCharValue(dbTransaction, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			       "date", GWEN_Buffer_GetStart(tbuf));
+        GWEN_Buffer_free(tbuf);
+      }
+
+      ti=AB_Transaction_GetValutaDate(t);
+      if (ti) {
+	GWEN_BUFFER *tbuf;
+
+	tbuf=GWEN_Buffer_new(0, 32, 0, 1);
+	if (inUtc)
+	  rv=GWEN_Time_toUtcString(ti, dateFormat, tbuf);
+	else
+	  rv=GWEN_Time_toString(ti, dateFormat, tbuf);
+	if (rv) {
+	  DBG_WARN(AQBANKING_LOGDOMAIN, "Bad date format string/date");
+	}
+	else
+	  GWEN_DB_SetCharValue(dbTransaction, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			       "valutaDate", GWEN_Buffer_GetStart(tbuf));
+	GWEN_Buffer_free(tbuf);
+      }
+
+      /* possibly transform value */
+      if (usePosNegField) {
+	const AB_VALUE *v;
+        const char *s;
+
+	v=AB_Transaction_GetValue(t);
+	if (v) {
+          if (!AB_Value_IsNegative(v)) {
+	    s=GWEN_DB_GetCharValue(params, "positiveValues", 0, 0);
+	    if (s) {
+	      GWEN_DB_SetCharValue(dbTransaction,
+				   GWEN_DB_FLAGS_OVERWRITE_VARS,
+				   posNegFieldName,
+				   s);
+	    }
+	    else {
+	      DBG_ERROR(AQBANKING_LOGDOMAIN,
+                        "No value for \"positiveValues\" in params");
+	      GWEN_DB_Group_free(dbData);
+	      return GWEN_ERROR_GENERIC;
+	    }
+	  }
+	  else {
+	    s=GWEN_DB_GetCharValue(params, "negativeValues", 0, 0);
+	    if (s) {
+	      AB_VALUE *nv;
+              GWEN_DB_NODE *dbV;
+
+	      GWEN_DB_SetCharValue(dbTransaction,
+				   GWEN_DB_FLAGS_OVERWRITE_VARS,
+				   posNegFieldName,
+				   s);
+	      nv=AB_Value_dup(v);
+	      AB_Value_Negate(nv);
+	      dbV=GWEN_DB_GetGroup(dbTransaction,
+				   GWEN_DB_FLAGS_OVERWRITE_GROUPS,
+				   "value");
+	      assert(dbV);
+	      if (AB_Value_toDb(nv, dbV)) {
+		DBG_ERROR(AQBANKING_LOGDOMAIN,
+			  "Could not store value to DB");
+		GWEN_DB_Group_free(dbData);
+		return GWEN_ERROR_GENERIC;
+	      }
+	    }
+	    else {
+	      DBG_ERROR(AQBANKING_LOGDOMAIN,
+			"No value for \"negativeValues\" in params");
+	      GWEN_DB_Group_free(dbData);
+	      return GWEN_ERROR_GENERIC;
+	    }
+	  }
+	}
+      }
+
+      /* add transaction db */
+      GWEN_DB_AddGroup(dbData, dbTransaction);
+      t=AB_ImExporterAccountInfo_GetNextNotedTransaction(ai);
+    }
+    ai=AB_ImExporterContext_GetNextAccountInfo(ctx);
+  }
+
+
+  rv=GWEN_DBIO_Export(ieh->dbio,
+		      io,
+		      dbData,
+		      dbSubParams,
+		      GWEN_DB_FLAGS_DEFAULT,
+		      guiid,
+                      2000);
+  if (rv) {
+    DBG_ERROR(AQBANKING_LOGDOMAIN, "Error exporting data (%d)", rv);
+    GWEN_Gui_ProgressLog(guiid, GWEN_LoggerLevel_Error,
+			 "Error exporting data");
+    GWEN_DB_Group_free(dbData);
+    return GWEN_ERROR_GENERIC;
+  }
+  GWEN_DB_Group_free(dbData);
+
+  return 0;
+}
+
+
+
+int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
+                            AB_IMEXPORTER_CONTEXT *ctx,
+                            GWEN_IO_LAYER *io,
+			    GWEN_DB_NODE *params,
+			    uint32_t guiid){
+  AH_IMEXPORTER_CSV *ieh;
+  const char *subject;
+
+  assert(ie);
+  ieh=GWEN_INHERIT_GETDATA(AB_IMEXPORTER, AH_IMEXPORTER_CSV, ie);
+  assert(ieh);
+  assert(ieh->dbio);
+
+  subject=GWEN_DB_GetCharValue(params, "subject", 0,
+			       "transactions");
+  if (strcasecmp(subject, "transactions")==0)
+    return AH_ImExporterCSV__ExportTransactions(ie, ctx, io, params, guiid);
+  else if (strcasecmp(subject, "notedTransactions")==0)
+    return AH_ImExporterCSV__ExportNotedTransactions(ie, ctx, io,
+						     params, guiid);
+  else {
+    DBG_ERROR(AQBANKING_LOGDOMAIN,
+	      "Unable to export unknown subject \"%s\"", subject);
+    return GWEN_ERROR_INVALID;
+  }
+}
+
+
 
 
