@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: swift940.c 1018 2006-05-15 17:43:16Z martin $
+ cvs         : $Id$
  begin       : Fri Apr 02 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -16,6 +16,7 @@
 #endif
 
 #include "swift940_p.h"
+#include "i18n_l.h"
 
 /* #include <aqhbci/aqhbci.h> */
 #include <aqbanking/error.h>
@@ -24,16 +25,15 @@
 #include <gwenhywfar/text.h>
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/gwentime.h>
-#include <gwenhywfar/waitcallback.h>
+#include <gwenhywfar/gui.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
 
 
-
 int AHB_SWIFT__SetCharValue(GWEN_DB_NODE *db,
-                            GWEN_TYPE_UINT32 flags,
+                            uint32_t flags,
                             const char *name,
                             const char *s) {
   GWEN_BUFFER *vbuf;
@@ -49,7 +49,7 @@ int AHB_SWIFT__SetCharValue(GWEN_DB_NODE *db,
 
 
 int AHB_SWIFT940_Parse_25(const AHB_SWIFT_TAG *tg,
-                          GWEN_TYPE_UINT32 flags,
+                          uint32_t flags,
                           GWEN_DB_NODE *data,
                           GWEN_DB_NODE *cfg){
   const char *p;
@@ -70,10 +70,12 @@ int AHB_SWIFT940_Parse_25(const AHB_SWIFT_TAG *tg,
     char *s;
 
     /* "BLZ/Konto" */
-    s=(char*)malloc(p2-p+1);
+    s=(char*)GWEN_Memory_malloc(p2-p+1);
     memmove(s, p, p2-p+1);
     s[p2-p]=0;
-    AHB_SWIFT__SetCharValue(data, flags, "localBankCode", s);
+    AHB_SWIFT__SetCharValue(data,
+                            GWEN_DB_FLAGS_OVERWRITE_VARS,
+                            "localBankCode", s);
     free(s);
     p=p2+1;
   }
@@ -88,15 +90,19 @@ int AHB_SWIFT940_Parse_25(const AHB_SWIFT_TAG *tg,
     if (p2==p) {
       DBG_WARN(AQBANKING_LOGDOMAIN,
                "LocalAccountNumber starts with nondigits (%s)", p);
-      AHB_SWIFT__SetCharValue(data, flags, "localAccountNumber", p);
+      AHB_SWIFT__SetCharValue(data,
+                              GWEN_DB_FLAGS_OVERWRITE_VARS,
+                              "localAccountNumber", p);
     }
     else {
       char *s;
 
-      s=(char*)malloc(p2-p+1);
+      s=(char*)GWEN_Memory_malloc(p2-p+1);
       memmove(s, p, p2-p+1);
       s[p2-p]=0;
-      AHB_SWIFT__SetCharValue(data, flags, "localAccountNumber", s);
+      AHB_SWIFT__SetCharValue(data,
+                              GWEN_DB_FLAGS_OVERWRITE_VARS,
+                              "localAccountNumber", s);
       free(s);
     }
   }
@@ -106,7 +112,7 @@ int AHB_SWIFT940_Parse_25(const AHB_SWIFT_TAG *tg,
 
 
 int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
-                          GWEN_TYPE_UINT32 flags,
+                          uint32_t flags,
                           GWEN_DB_NODE *data,
                           GWEN_DB_NODE *cfg){
   const char *p;
@@ -139,7 +145,7 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
     while(*p) {
       if (strlen(p)<3) {
 	DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad field in :86: tag (%s)", p);
-        GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+	GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                               "SWIFT: Bad field in :86: tag");
         return -1;
       }
@@ -149,7 +155,7 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
         p++;
       if (!*p) {
 	DBG_ERROR(AQBANKING_LOGDOMAIN, "Partial field id");
-        GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                               "SWIFT: Partial field id");
         return -1;
       }
@@ -160,7 +166,7 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
         p++;
       if (!*p) {
         DBG_ERROR(AQBANKING_LOGDOMAIN, "Partial field id");
-        GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                               "SWIFT: Partial field id");
         return -1;
       }
@@ -171,7 +177,7 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
 
       /* find end of field/beginning of next field */
       while(*p2 && *p2!='?') p2++;
-      s=(char*)malloc(p2-p+1);
+      s=(char*)GWEN_Memory_malloc(p2-p+1);
       memmove(s, p, p2-p+1);
       s[p2-p]=0;
       AHB_SWIFT_Condense(s);
@@ -244,7 +250,7 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
 
 
 int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
-                          GWEN_TYPE_UINT32 flags,
+                          uint32_t flags,
                           GWEN_DB_NODE *data,
                           GWEN_DB_NODE *cfg){
   const char *p;
@@ -266,7 +272,7 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   /* valuata date (M) */
   if (bleft<6) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Missing valuta date (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Missing valuta date");
     return -1;
   }
@@ -274,8 +280,21 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   d1a+=2000;
   d2a=((p[2]-'0')*10) + (p[3]-'0');
   d3a=((p[4]-'0')*10) + (p[5]-'0');
-  ti=GWEN_Time_new(d1a, d2a-1, d3a, 12, 0, 0, 1);
-  assert(ti);
+
+  if (d3a==30 && d2a==2) {
+    /* date is Feb 30, this date is invalid. However, some banks use this
+     * to indicate the last day of February, so we move along */
+    d3a=1;
+    d2a=3;
+    ti=GWEN_Time_new(d1a, d2a-1, d3a, 12, 0, 0, 1);
+    assert(ti);
+    /* subtract a day to get the last day in FEB */
+    GWEN_Time_SubSeconds(ti, 60*60*24);
+  }
+  else {
+    ti=GWEN_Time_new(d1a, d2a-1, d3a, 12, 0, 0, 1);
+    assert(ti);
+  }
   if (GWEN_Time_toDb(ti, GWEN_DB_GetGroup(data,
                                           GWEN_DB_FLAGS_OVERWRITE_GROUPS,
                                           "valutadate"))) {
@@ -284,11 +303,11 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   p+=6;
   bleft-=6;
 
-  /* booking data (K) */
+  /* booking date (K) */
   if (*p && isdigit(*p)) {
     if (bleft<4) {
       DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad booking date (%s)", p);
-      GWEN_WaitCallback_Log(GWEN_LoggerLevelInfo,
+      GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Info,
                             "SWIFT: Bad booking date");
       return -1;
     }
@@ -326,7 +345,7 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   /* credit/debit mark (M) */
   if (bleft<2) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad value string (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Bad value string");
     return -1;
   }
@@ -347,7 +366,7 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   /* third character of currency (K) */
   if (bleft<1) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad data (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Bad currency");
     return -1;
   }
@@ -364,11 +383,11 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
     p2++;
   if (p2==p) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "No value (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Bad value");
     return -1;
   }
-  s=(char*)malloc(p2-p+1+(neg?1:0));
+  s=(char*)GWEN_Memory_malloc(p2-p+1+(neg?1:0));
   if (neg) {
     s[0]='-';
     memmove(s+1, p, p2-p+1);
@@ -394,13 +413,13 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   /* key (M) */
   if (bleft<3) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Missing booking key (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Missing booking key");
     return -1;
   }
   memmove(buffer, p, 3);
   buffer[3]=0;
-  AHB_SWIFT__SetCharValue(data, flags, "key", buffer);
+  AHB_SWIFT__SetCharValue(data, flags, "transactionKey", buffer);
   p+=3;
   bleft-=3;
 
@@ -416,11 +435,11 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
 
     if (p2==p) {
       DBG_WARN(AQBANKING_LOGDOMAIN, "Missing customer reference (%s)", p);
-      GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+      GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
 			    "SWIFT: Missing customer reference");
     }
     else {
-      s=(char*)malloc(p2-p+1);
+      s=(char*)GWEN_Memory_malloc(p2-p+1);
       memmove(s, p, p2-p);
       s[p2-p]=0;
       if (strcasecmp(s, "NONREF")!=0)
@@ -443,11 +462,11 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
       while(*p2 && *p2!='/' && *p2!=10) p2++;
       if (p2==p) {
 	DBG_ERROR(AQBANKING_LOGDOMAIN, "Missing bank reference (%s)", p);
-	GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+	GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
 			      "SWIFT: Missing bank reference");
 	return -1;
       }
-      s=(char*)malloc(p2-p+1);
+      s=(char*)GWEN_Memory_malloc(p2-p+1);
       memmove(s, p, p2-p+1);
       s[p2-p]=0;
       AHB_SWIFT__SetCharValue(data, flags, "bankReference", s);
@@ -490,11 +509,11 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
           while(*p2 && *p2!='/') p2++;
           if (p2==p) {
             DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad original value (%s)", p);
-            GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+            GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                                   "SWIFT: Bad original value");
             return -1;
           }
-          s=(char*)malloc(p2-p+1);
+          s=(char*)GWEN_Memory_malloc(p2-p+1);
           memmove(s, p, p2-p+1);
           s[p2-p]=0;
           AHB_SWIFT__SetCharValue(data, flags, "origvalue/value", s);
@@ -517,11 +536,11 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
           while(*p2 && *p2!='/') p2++;
           if (p2==p) {
             DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad charges value (%s)", p);
-            GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+            GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                                   "SWIFT: Bad charges value");
             return -1;
           }
-          s=(char*)malloc(p2-p+1);
+	  s=(char*)GWEN_Memory_malloc(p2-p+1);
           memmove(s, p, p2-p+1);
           s[p2-p]=0;
           AHB_SWIFT__SetCharValue(data, flags, "charges/value", s);
@@ -548,7 +567,7 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
 
 
 int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
-                             GWEN_TYPE_UINT32 flags,
+                             uint32_t flags,
                              GWEN_DB_NODE *data,
                              GWEN_DB_NODE *cfg){
   const char *p;
@@ -567,7 +586,7 @@ int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
   /* credit/debit mark (M) */
   if (bleft<2) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad value string (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Bad value string");
     return -1;
   }
@@ -580,7 +599,7 @@ int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
   /* date (M) */
   if (bleft<6) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Missing date (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Missing date");
     return -1;
   }
@@ -606,7 +625,7 @@ int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
      * swiss MT940) */
     if (bleft<3) {
       DBG_ERROR(AQBANKING_LOGDOMAIN, "Missing currency (%s)", p);
-      GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+      GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
 			    "SWIFT: Missing currency");
       return -1;
     }
@@ -620,7 +639,7 @@ int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
   /* value (M) */
   if (bleft<1) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Missing value (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
 			  "SWIFT: Missing value");
     return -1;
   }
@@ -629,11 +648,11 @@ int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
   while(*p2 && (isdigit(*p2) || *p2==',')) p2++;
   if (p2==p) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad value (%s)", p);
-    GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                           "SWIFT: Bad value");
     return -1;
   }
-  s=(char*)malloc(p2-p+1+(neg?1:0));
+  s=(char*)GWEN_Memory_malloc(p2-p+1+(neg?1:0));
   if (neg) {
     s[0]='-';
     memmove(s+1, p, p2-p+1);
@@ -654,7 +673,7 @@ int AHB_SWIFT940_Parse_6_0_2(const AHB_SWIFT_TAG *tg,
 
 
 int AHB_SWIFT940_Parse_NS(const AHB_SWIFT_TAG *tg,
-                          GWEN_TYPE_UINT32 flags,
+                          uint32_t flags,
                           GWEN_DB_NODE *data,
                           GWEN_DB_NODE *cfg){
   const char *p;
@@ -695,7 +714,7 @@ int AHB_SWIFT940_Parse_NS(const AHB_SWIFT_TAG *tg,
       else {
 	char *s;
 
-	s=(char*)malloc(len+1);
+	s=(char*)GWEN_Memory_malloc(len+1);
 	memmove(s, p, len);
 	s[len]=0;
 	DBG_DEBUG(AQBANKING_LOGDOMAIN, "Got his field: %02d: %s", code, s);
@@ -762,24 +781,29 @@ int AHB_SWIFT940_Parse_NS(const AHB_SWIFT_TAG *tg,
 
 
 
-int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
-                        AHB_SWIFT_TAG_LIST *tl,
-                        GWEN_TYPE_UINT32 flags,
-                        GWEN_DB_NODE *data,
-                        GWEN_DB_NODE *cfg){
+int AHB_SWIFT940_Import(AHB_SWIFT_TAG_LIST *tl,
+			GWEN_DB_NODE *data,
+			GWEN_DB_NODE *cfg,
+			uint32_t flags,
+			uint32_t guiid,
+			int msecs) {
   AHB_SWIFT_TAG *tg;
-  GWEN_DB_NODE *dbDay;
-  GWEN_DB_NODE *dbTemplate;
-  GWEN_DB_NODE *dbTransaction;
-  int tagCount;
-
-  dbDay=0;
-  dbTransaction=0;
-  tagCount=0;
+  GWEN_DB_NODE *dbDay=NULL;
+  GWEN_DB_NODE *dbTemplate=NULL;
+  GWEN_DB_NODE *dbTransaction=NULL;
+  GWEN_DB_NODE *dbDate=NULL;
+  uint32_t progressId;
 
   dbTemplate=GWEN_DB_Group_new("template");
 
-  GWEN_WaitCallback_SetProgressTotal(AHB_SWIFT_Tag_List_GetCount(tl));
+  progressId=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+				    GWEN_GUI_PROGRESS_ALLOW_EMBED |
+				    GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+				    GWEN_GUI_PROGRESS_SHOW_ABORT,
+				    I18N("Importing SWIFT tags..."),
+				    NULL,
+				    AHB_SWIFT_Tag_List_GetCount(tl),
+				    guiid);
 
   tg=AHB_SWIFT_Tag_List_First(tl);
   while(tg) {
@@ -792,6 +816,7 @@ int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
       if (AHB_SWIFT940_Parse_25(tg, flags, dbTemplate, cfg)) {
         DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
         GWEN_DB_Group_free(dbTemplate);
+	GWEN_Gui_ProgressEnd(progressId);
         return -1;
       }
     }
@@ -810,15 +835,19 @@ int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
       if (AHB_SWIFT940_Parse_6_0_2(tg, flags, dbSaldo, cfg)) {
         DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
         GWEN_DB_Group_free(dbTemplate);
+	GWEN_Gui_ProgressEnd(progressId);
         return -1;
       }
+      else {
+	dbDate=GWEN_DB_GetGroup(dbSaldo, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+				"date");
+      }
 
       curr=GWEN_DB_GetCharValue(dbSaldo, "value/currency", 0, 0);
       if (curr) {
 	AHB_SWIFT__SetCharValue(dbTemplate, flags,
 				"value/currency", curr);
       }
-
     }
     else if (strcasecmp(id, "62F")==0) { /* EndSaldo */
       GWEN_DB_NODE *dbSaldo;
@@ -835,6 +864,7 @@ int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
       if (AHB_SWIFT940_Parse_6_0_2(tg, flags, dbSaldo, cfg)) {
         DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
         GWEN_DB_Group_free(dbTemplate);
+	GWEN_Gui_ProgressEnd(progressId);
         return -1;
       }
       dbDay=0;
@@ -850,9 +880,20 @@ int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
       dbTransaction=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
                                      "transaction");
       GWEN_DB_AddGroupChildren(dbTransaction, dbTemplate);
+      if (dbDate) {
+	GWEN_DB_NODE *dbT;
+
+	/* dbDate is set upon parsing of tag 60F, use it as a default
+	 * if possible */
+	dbT=GWEN_DB_GetGroup(dbTransaction, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
+			     "date");
+        assert(dbT);
+	GWEN_DB_AddGroupChildren(dbT, dbDate);
+      }
       if (AHB_SWIFT940_Parse_61(tg, flags, dbTransaction, cfg)) {
         DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
         GWEN_DB_Group_free(dbTemplate);
+	GWEN_Gui_ProgressEnd(progressId);
         return -1;
       }
     }
@@ -865,6 +906,7 @@ int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
         if (AHB_SWIFT940_Parse_86(tg, flags, dbTransaction, cfg)) {
           DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
           GWEN_DB_Group_free(dbTemplate);
+	  GWEN_Gui_ProgressEnd(progressId);
           return -1;
         }
       }
@@ -878,22 +920,26 @@ int AHB_SWIFT940_Import(GWEN_BUFFEREDIO *bio,
 	if (AHB_SWIFT940_Parse_NS(tg, flags, dbTransaction, cfg)) {
 	  DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
 	  GWEN_DB_Group_free(dbTemplate);
+	  GWEN_Gui_ProgressEnd(progressId);
 	  return -1;
 	}
       }
     }
 
-    tagCount++;
-    if (GWEN_WaitCallbackProgress(tagCount)==GWEN_WaitCallbackResult_Abort){
-      DBG_INFO(AQBANKING_LOGDOMAIN, "User aborted");
-      GWEN_WaitCallback_Log(GWEN_LoggerLevelError,
-                            "SWIFT: User aborted");
+    if (GWEN_Gui_ProgressAdvance(progressId, GWEN_GUI_PROGRESS_ONE)==
+	GWEN_ERROR_USER_ABORTED) {
+      GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
+			   I18N("Aborted by user"));
+      GWEN_Gui_ProgressEnd(progressId);
       GWEN_DB_Group_free(dbTemplate);
-      return AB_ERROR_USER_ABORT;
+      return GWEN_ERROR_USER_ABORTED;
     }
+
     tg=AHB_SWIFT_Tag_List_Next(tg);
   } /* while */
+
   GWEN_DB_Group_free(dbTemplate);
+  GWEN_Gui_ProgressEnd(progressId);
 
   return 0;
 }
