@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
  -------------------
- cvs         : $Id: version.h.in 269 2005-04-07 16:50:57Z aquamaniac $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -14,19 +14,19 @@
 #ifndef AQBANKING_VERSION_H
 #define AQBANKING_VERSION_H
 
-#define AQBANKING_VERSION_MAJOR 2
-#define AQBANKING_VERSION_MINOR 2
-#define AQBANKING_VERSION_PATCHLEVEL 3
+#define AQBANKING_VERSION_MAJOR 3
+#define AQBANKING_VERSION_MINOR 5
+#define AQBANKING_VERSION_PATCHLEVEL 1
 #define AQBANKING_VERSION_BUILD 0
 #define AQBANKING_VERSION_TAG "stable"
-#define AQBANKING_VERSION_FULL_STRING "2.2.3.0-stable"
-#define AQBANKING_VERSION_STRING "2.2.3"
+#define AQBANKING_VERSION_FULL_STRING "3.5.1.0stable"
+#define AQBANKING_VERSION_STRING "3.5.1"
 
 
-#define AQBANKING_SO_CURRENT 19
-#define AQBANKING_SO_REVISION 2
-#define AQBANKING_SO_AGE 3
-#define AQBANKING_SO_EFFECTIVE 16
+#define AQBANKING_SO_CURRENT 25
+#define AQBANKING_SO_REVISION 0
+#define AQBANKING_SO_AGE 5
+#define AQBANKING_SO_EFFECTIVE 20
 
 #endif /* AQBANKING_VERSION_H */
 
